/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.action.support;

import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.inject.Inject;
import javax.ws.rs.Path;

import org.loom.action.AbstractAction;
import org.loom.annotation.Cache;
import org.loom.annotation.Event;
import org.loom.annotation.SSLPolicy;
import org.loom.annotation.SSLPolicyType;
import org.loom.exception.HttpException;
import org.loom.i18n.MessagesRepository;
import org.loom.resolution.AbstractHttpResolution;
import org.loom.resolution.InputStreamResolution;
import org.loom.resolution.Resolution;
import org.loom.resolution.StringResolution;
import org.loom.resources.CompiledWebResource;
import org.loom.resources.CssResource;
import org.loom.resources.WebResourceBundle;
import org.loom.resources.WebResourceBundleRepository;
import org.loom.servlet.CacheControl;
import org.loom.servlet.LoomServletRequestFactory;
import org.loom.servlet.names.ContentTypeNames;
import org.loom.servlet.names.RequestHeaderNames;
import org.loom.servlet.names.ResponseHeaderNames;


/**
 * Retrieves a static resource, such as a javascript/css file.
 * It will set the ETag as the MD5 of the response contents.
 * 
 * @author icoloma
 */
@SSLPolicy(SSLPolicyType.DO_NOT_MODIFY)
public class ResourcesAction extends AbstractAction {

	@Inject
	private WebResourceBundleRepository webResourceBundleRepository;
	
	@Inject
	private LoomServletRequestFactory loomServletRequestFactory;
	
	/** 
	 * the name of the resource GET parameter. 
	 * This parameter is informational for the user (to see the script name in the browser),
	 * and will never be used by the server for security reasons 
	 */
	public static final String RESOURCE = "resourceName";
	
	/** the resource bundle name */
	public static final String BUNDLE = "bundle";
	
	/** the name of the md5 GET parameter */
	public static final String MD5 = "md5";
	
	/** if present, use this MD5 to check if the browser data is up-to-date */
	private String md5;

	/** the javascript/css/image file to return */
	private String resourceName;
	
	/** used when invoking getResource() */
	private String bundle;
	
	/** the resolved resource bundle */
	private WebResourceBundle webResourceBundle;
	
	/** when getResource() is invoked, the resolved resource (js/css file) */
	private CompiledWebResource compiledWebResource;
	
	/** when getCssResource() is invoked, the external resource instance */
	private CssResource cssResource;
	
	/**
	 * If a MD5 attribute has been passed and it's the same of the actual MessageRepository contents,
	 * set the response to cache forever.
	 * If no MD5 GET parameter has been passed, use it as ETag as usual.
	 */
	@Cache(on="getJsMessagesRepository")
	public CacheControl cacheMessagesRepository() {
		CacheControl c = new CacheControl();
		MessagesRepository repository = getContext().getRequest().getMessagesRepository();
		c.withEtag(repository.getMD5());
		if (md5 != null && md5.equals(c.getEtag())) {
			c.forever();
		}
		return c;
	}
	
	/**
	 * Converts the {@link MessagesRepository} instance to a javascript JSON presentation.
	 * If the provided MD5 parameter is the same of the current one, the response will be set 
	 * to cache forever.
	 * @return a javascript version of the {@link MessagesRepository} for this locale
	 */
	@Event(defaultEvent=true)
	@Path("/i18n/{md5}")
	public Resolution getJsMessagesRepository() {
		MessagesRepository messagesRepository = getMessagesRepository();
		AbstractHttpResolution r;
		if (supportsGzip()) {
			r = new InputStreamResolution(new ByteArrayInputStream(messagesRepository.getGzippedJson()));
			r.setHeader(ResponseHeaderNames.CONTENT_ENCODING, "gzip");
		} else {
			r = new StringResolution(messagesRepository.toJSON());
		}
		r.setContentType(ContentTypeNames.JSON);
		r.setCharset("UTF-8");
		return r;
	}
	
	@Cache(on="getResource")
	public CacheControl cacheResource() throws IOException {
		CacheControl c = new CacheControl();
		webResourceBundle = webResourceBundleRepository.getWebResourceBundle(bundle);
		compiledWebResource = webResourceBundle.getCompiledWebResource(resourceName == null? WebResourceBundle.PROD_RESOURCE_NAME : resourceName);
		c.withEtag(compiledWebResource.getMd5());
		if (md5 != null && md5.equals(c.getEtag())) {
			c.forever();
		}
		return c;
	}
	
	/**
	 * If Config.development = false, retrieves an entire concatenated bundle
	 * and returns it gzipped (if the browser supports gzip).
	 * @return the requested resource, gzipped if supported by the client
	 */
	@Path("/{bundle}/{md5}/{resourceName?}")
	public Resolution getResource() throws FileNotFoundException {
		InputStreamResolution resolution;
		if (supportsGzip()) { // accepts gzip
			resolution = new InputStreamResolution(null, compiledWebResource.getGzippedInputStream(), compiledWebResource.getGzippedLength());
			resolution.setHeader(ResponseHeaderNames.CONTENT_ENCODING, "gzip");
		} else {
			resolution = new InputStreamResolution(null, compiledWebResource.getInputStream(), compiledWebResource.getLength());
		}
		resolution.setContentType(webResourceBundle.getType().getContentType());
		resolution.setCharset(loomServletRequestFactory.getDefaultCharset());
		return resolution;
	}
	
	
	@Cache(on="getCssResource")
	public CacheControl cacheCssResource() throws IOException {
		CacheControl c = new CacheControl();
		webResourceBundle = webResourceBundleRepository.getWebResourceBundle(bundle);
		cssResource = webResourceBundle.getCssResource(resourceName);
		c.withEtag(cssResource.getMd5());
		if (md5 != null && md5.equals(c.getEtag())) {
			c.forever();
		}
		return c;
	}
	
	/**
	 * @return images referenced from the CSS file
	 */
	@Path("/{bundle}/_res/{md5}/{resourceName?}")
	public Resolution getCssResource() throws IOException {
		if (!cssResource.getResource().exists()) {
			throw new HttpException(404, "CSS resource referenced by bundle " + bundle + " not found: " + cssResource.getResource().getDescription());
		}
		InputStreamResolution resolution = new InputStreamResolution(null, cssResource.getResource().getInputStream());
		resolution.setContentType(cssResource.getContentType());
		return resolution;
	}
	
	private boolean supportsGzip() {
		String header = getContext().getRequest().getHeader(RequestHeaderNames.ACCEPT_ENCODING);
		return header != null && header.contains("gzip");
	}
	
}
