/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.binding;

import org.loom.exception.ConfigException;
import org.loom.exception.MalformedPropertyPathException;
import org.loom.util.PropertyUtils;
import org.loom.validator.ValidationRequest;

public abstract class AbstractCollectionNode extends Node {

	@Override
	public Object getValue(Object container, String[] propertyPath, int propertyIndex) {
		// if this is an end node, the value will be assigned
		int listIndex = getIndexAsInt(propertyPath[propertyIndex]);
		if (propertyIndex == propertyPath.length - 1)  {
			Object propertyList = getPropertyWrapper().get(container, false);
			if (propertyList == null)
				return null;
			
			// if no index is specified, return the entire list
			return listIndex == -1? propertyList: get(propertyList, listIndex);
			
		} else {
			
			// this is not an end node. Traverse it.
			Object propertyList = getPropertyWrapper().get(container, true);
			Object item = addItemToList(container, propertyList, propertyPath, propertyIndex, listIndex);
			return getChild().getValue(item, propertyPath, propertyIndex + 1);
			
		}

	}


	@Override
	public void setValue(Object container, Object value, String[] propertyPath, int index) {
		// if this is an end node, the list of values will be converted and assigned
		Object propertyList = getPropertyWrapper().get(container, true);
		String nodeName = propertyPath[index];
		int listIndex = getIndexAsInt(nodeName);
		if (index == propertyPath.length - 1) { 
			if (listIndex == -1) { //  missing index, assign the whole list property
				getPropertyWrapper().set(container, value);
			} else if (value != null) {
				propertyList = ensureCapacity(container, propertyList, listIndex + 1);
				set(propertyList, listIndex, value);
			}
		} else {
			// this is not an end node. Traverse the node
			Object item = addItemToList(container, propertyList, propertyPath, index, listIndex);
			getChild().setValue(item, value, propertyPath, index + 1);
		}
		
	}
	
	/**
	 * Creates and add an item to the list if there is none there.
	 * If there is already a value in that position, do nothing and return it.
	 * @param propertyList the list of values of this property
	 * @param propertyPath the property path
	 * @param propertyIndex the current node index
	 * @return the item added to the list
	 */
	protected Object addItemToList(Object container, Object propertyList, String[] propertyPath, int propertyIndex, int listIndex) {
		if (listIndex == -1)
			throw new MalformedPropertyPathException("Missing index at node " + propertyPath[propertyIndex]);
		propertyList = ensureCapacity(container, propertyList, listIndex + 1);
		Object item = get(propertyList, listIndex);
		if (item == null) {
			try {
				item = getPropertyWrapper().getItemMetadata().newItem();
				set(propertyList, listIndex, item);
			} catch (ConfigException e) {
				throw new ConfigException("Cannot navigate through property " + PropertyUtils.concatPropertyPath(propertyPath) + ". Please specify a @CollectionMetadata.itemClass.", e);
			}
		}
		return item;
	}
	
	/**
	 * Checks that the collection has at least the expected size. Enlarge it if it doesn't
	 * @param container the instance that contains this collection property
	 * @param collection the collection to check
	 * @param minSize the minimum size required
	 * @return the original collection if it does have the minimum size required, or a  
	 * bigger copy if not.
	 */
	protected abstract Object ensureCapacity(Object container, Object collection, int minSize);
	
	@Override
	protected void applyValidationImpl(Object container, ValidationRequest request) {
		Object collection = getPropertyWrapper().get(container);
		if (isLeaf() && request.getValidator().validateCollectionContainer()) {
			request.push(getPropertyName());
			request.validate(collection);
			request.pop();
		} else if (collection != null) {
			int collectionSize = getSize(collection);
			for (int i = 0; i < collectionSize; i++) {
				Object value = get(collection, i);
				request.push(getPropertyName(), i);
				if (isLeaf()) {
					request.validate(value);
				} else {
					getChild().applyValidation(value, request);
				}
				request.pop();
			}
		}
	}


	/**
	 * @param list the List/array instance
	 * @return the size of the list
	 */
	protected abstract int getSize(Object list);
	
	/**
	 * @return the required item in the list
	 */
	protected abstract Object get(Object list, int index);
	
	/**
	 * Adds an item to the list
	 */
	protected abstract void set(Object list, int index, Object item);

	@Override
	public void setPropertyWrapper(PropertyWrapper wrapper) {
		assert wrapper.getItemMetadataSize() == 1 : "Expected one ItemMetadata instance";
		super.setPropertyWrapper(wrapper);
	}

}
