/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.binding;

import java.util.Map;

import org.loom.converter.LocaleUnawareConverter;
import org.loom.exception.ConfigException;
import org.loom.exception.MalformedPropertyPathException;
import org.loom.util.PropertyUtils;
import org.loom.validator.ValidationRequest;


/**
 * Map
 * @author Ignacio Coloma
 *
 */
public class MapNode extends Node {

	@Override
	public Object getValue(Object container, String[] propertyPath, int propertyIndex) {
		// if this is an end node, the list of values will be converted an assigned
		Map<?, ?> propertyMap = (Map<?, ?>) getPropertyWrapper().get(container, true);
		Object key = getConvertedKey(propertyPath[propertyIndex]);
		if (propertyIndex == propertyPath.length - 1)  {
			return key == null? propertyMap : propertyMap.get(key);
		} else {
			// this is not an end node. Traverse it.
			Object item = addItemToMap(propertyPath, propertyIndex, propertyMap, key);
			return getChild().getValue(item, propertyPath, propertyIndex + 1);
		}

	}
	
	private Object getConvertedKey(String propertyName) {
		String keyAsString = getIndex(propertyName);
		LocaleUnawareConverter converter = (LocaleUnawareConverter) getPropertyWrapper().getItemMetadata(0).getConverter();
		return converter.getAsObject(keyAsString);
	}

	@SuppressWarnings("unchecked")
	@Override
	public void setValue(Object container, Object value, String[] propertyPath, int index) {
		// if this is an end node, the list of values will be converted and assigned
		Map propertyMap = (Map) getPropertyWrapper().get(container, true);
		Object key = getConvertedKey(propertyPath[index]);
		if (index == propertyPath.length - 1) { 
			if (key == null) {
				getPropertyWrapper().set(container, value);
			} else if (value != null) {
				propertyMap.put(key, value);
			}
		} else {
			// this is not an end node. Traverse the node
			Object item = addItemToMap(propertyPath, index, propertyMap, key);
			getChild().setValue(item, value, propertyPath, index + 1);
		}
		
	}
	
	/**
	 * Creates and puts a map value.
	 * If there is already a value with that key, do nothing and return it.
	 * @param propertyMap the map to manipulate
	 * @param key the key
	 * @return the item added to the map
	 */
	@SuppressWarnings("unchecked")
	private Object addItemToMap(String[] propertyPath, int propertyIndex, Map propertyMap, Object key) {
		if (key == null) {
			throw new MalformedPropertyPathException("Expected key at " + propertyPath[propertyIndex]);
		}
		Object value = propertyMap.get(key);
		if (value == null) {
			try {
				value = getPropertyWrapper().getItemMetadata(1).newItem();
				propertyMap.put(key, value);
			} catch (ConfigException e) {
				throw new ConfigException("Cannot navigate through property " + PropertyUtils.concatPropertyPath(propertyPath) + ". Please specify a @MapMetadata.valueClass.", e);
			}

		}
		return value;
	}
	
	@Override
	protected void applyValidationImpl(Object container, ValidationRequest request) {
		Map<?, ?> map = (Map<?, ?>)getPropertyWrapper().get(container);
		if (isLeaf() && request.getValidator().validateCollectionContainer()) {
			// validate the map instead of its contents
			request.push(getPropertyName());
			request.validate(map);
			request.pop();
		} else {
			for (Map.Entry<?, ?> entry : map.entrySet()) {
				Object key = entry.getKey();
				Object value = entry.getValue();
				request.push(getPropertyName(), key);
				if (isLeaf()) {
					request.validate(value);
				} else {
					getChild().applyValidation(value, request);
				}
				request.pop();
			}
		}
	}

	@Override
	public void setPropertyWrapper(PropertyWrapper wrapper) {
		assert wrapper.getItemMetadataSize() == 2 : "Expected two ItemMetadata instances";
		super.setPropertyWrapper(wrapper);
	}

}
