/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.binding;

import javax.inject.Inject;
import javax.inject.Singleton;

import org.apache.commons.lang.StringUtils;
import org.loom.converter.Converter;
import org.loom.converter.ConverterFactory;


/**
 * The PropertyBinder factory.
 * @author Ignacio Coloma
 */
@Singleton
public class PropertyBinderFactory {

	/** there must be constructed a chain of Node to arrive to the leaf property through the propertyPath */
	@Inject
	private NodeFactory nodeFactory;
	
	@Inject
	private ConverterFactory converterFactory;
	
	public PropertyBinderFactory() {}
	
	public PropertyBinder create(Class<?> clazz, String normalizedPropertyPath) {
		if (StringUtils.isEmpty(normalizedPropertyPath)) {
			throw new IllegalArgumentException("Empty propertyPath passed to create()");
		}
		PropertyBinder binder = new PropertyBinder(normalizedPropertyPath);
		Node rootNode = nodeFactory.create(clazz, normalizedPropertyPath);
		binder.setRootNode(rootNode);
		PropertyWrapper propertyWrapper = binder.getLeafNode().getPropertyWrapper();
		Converter converter = converterFactory.getConverter(propertyWrapper);
		
		// this does not fit well into ConverterFactory, as it talks about nodes instead of the container object.
		// TODO: handling things this way does not seem like the right way
		if (converter == null) {
			if (propertyWrapper.isSet()) {
				converter = converterFactory.getConverterForClass(Boolean.class);
			} else if (propertyWrapper.isCollection()) {
				converter = propertyWrapper.getItemMetadata().getConverter();
			} else if (propertyWrapper.isMap()) {
				converter = propertyWrapper.getItemMetadata(1).getConverter();
			} 
		}
		binder.setConverter(converter);
		return binder;
	}

	/**
	 * Creates a PropertyBinderMap with all the property binders for a 
	 * class. These PropertyBinder will be used to assign and get values of 
	 * each requested property path.
	 * @param containerClass The class that is root of any property path
	 */
	public PropertyBinderMap createPropertyBinderMap(Class<?> containerClass) {
		PropertyBinderMap map = new PropertyBinderMap();
		map.setPropertyBinderFactory(this);
		map.setContainerClass(containerClass);
		return map;
	}
	
	public void setNodeFactory(NodeFactory nodeFactory) {
		this.nodeFactory = nodeFactory;
	}

	public void setConverterFactory(ConverterFactory converterFactory) {
		this.converterFactory = converterFactory;
	}

}
