/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.binding;


import java.util.Set;

import org.loom.converter.Converter;
import org.loom.converter.LocaleUnawareConverter;
import org.loom.log.Log;
import org.loom.validator.ValidationRequest;

/**
 * Node that corresponds with a Set property.
 * <strong>Set properties cannot be traversed</strong>. Since there is no way 
 * to identify a concrete node inside a Set, property paths like child.friends[1].name
 * are not allowed if friends is a Set (since there is no way to use the index in any meaningful way).
 * Sets can be used to identify "bags of things", provided that these are simple objects (Strings, Integers, etc)
 * and are tipically handled by inputCheckbox tags. 
 * @author Ignacio Coloma
 *
 */
public class SetNode extends Node {
	
	private static Log log = Log.getLog(SetNode.class);
	
	@Override
	@SuppressWarnings("unchecked")
	public void setValue(Object container, Object value, String[] propertyPath, int index) {
		String nodeName = propertyPath[index];
		log.trace("Visiting: ", nodeName);
		String stringIndex = getIndex(nodeName);
		if (stringIndex == null) {
			// if the index is not present, assign the entire Set
			getPropertyWrapper().set(container, value);
		} else {
			// else, add a Set element
			Set set = (Set) getPropertyWrapper().get(container, true);
			LocaleUnawareConverter converter = getConverter();
			Object indexObject = (converter).getAsObject(stringIndex);
			if (Boolean.TRUE.equals(value)) {
				set.add(indexObject);
			} else {
				set.remove(indexObject);
			}
		}
	}

	private LocaleUnawareConverter getConverter() {
		Converter converter = getPropertyWrapper().getItemMetadata().getConverter();
		if (converter == null || !(converter instanceof LocaleUnawareConverter)) {
			throw new IllegalArgumentException("Items of Set " + getPropertyName() + " require a LocaleUnawareConverter. Found converter: " + converter);
		}
		return (LocaleUnawareConverter) converter;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public Object getValue(Object container, String[] propertyPath, int propertyIndex) {
		String stringIndex = getIndex(propertyPath[propertyIndex]);
		if (stringIndex == null) {
			return getPropertyWrapper().get(container);
		} else {
			Set set = (Set) getPropertyWrapper().get(container, true);
			LocaleUnawareConverter converter = getConverter();
			Object indexObject = (converter).getAsObject(stringIndex);
			return set.contains(indexObject);
		}
			
	}
	
	@Override
	protected void applyValidationImpl(Object container, ValidationRequest request) {
		Object value = getPropertyWrapper().get(container);
		request.validate(value);
	}
	
	@Override
	public void setPropertyWrapper(PropertyWrapper wrapper) {
		assert wrapper.getItemMetadataSize() == 1 : "Expected one ItemMetadata instance";
		super.setPropertyWrapper(wrapper);
	}
	
	@Override
	public void setChild(Node child) {
		if (child != null) {
			throw new IllegalArgumentException("Cannot append child node '" + getPropertyName() + "." + child.getPropertyName() + "'. Sets must be end nodes of a property path.");
		}
		super.setChild(child);
	}
}
