/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.binding.factories;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.inject.Singleton;

import org.loom.annotation.ImplementationMetadata;
import org.loom.binding.PropertyWrapper;
import org.loom.util.ClassUtils;


/**
 * An ordered list of implementation repositories: stops when one registered 
 * repository has a valid Factory for a property.
 * If none can, returns a DefaultImplementationFactory
 * @author Ignacio Coloma
 *
 */
@Singleton
public class ImplementationFactoriesRepository {

	/** default list implementation class */
	private ImplementationFactory<?> listImplementationFactory = new DefaultImplementationFactory(ArrayList.class);
	
	/** default map implementation class */
	private ImplementationFactory<?> mapImplementationFactory = new DefaultImplementationFactory(HashMap.class);
	
	/** default set implementation class */
	private ImplementationFactory<?> setImplementationFactory = new DefaultImplementationFactory(HashSet.class);
	
	/** default sorted set implementation class */
	private ImplementationFactory<?> sortedSetImplementationFactory = new DefaultImplementationFactory(TreeSet.class);
	
	/**
	 * @return the implementation factory for this property, null if none is specified and 
	 * it could not be guessed
	 */
	public ImplementationFactory<?> getImplementationFactory(PropertyWrapper wrapper) {
		
		// ImplementationMetadata-annotated fields
		ImplementationMetadata annotation = wrapper.getAnnotation(ImplementationMetadata.class);
		if (annotation != null) {
			return new DefaultImplementationFactory(annotation.value());
		}

		return getImplementationFactory(wrapper.getPropertyClass());
		
	}
	
	/**
	 * @return the default ImplementationFactory for the given class, null if it is not recognized
	 */
	public ImplementationFactory<?> getImplementationFactory(Class<?> propertyClass) {
		
		// ORDER IS IMPORTANT!!
		
		// array
		if (propertyClass.isArray()) {
			return new ArrayImplementationFactory(propertyClass.getComponentType());
		}
		
		// list and map
		if (List.class.equals(propertyClass)) {
			return listImplementationFactory;
		}
		if (Map.class.equals(propertyClass)) {
			return mapImplementationFactory;
		}
		if (SortedSet.class.equals(propertyClass)) {
			return sortedSetImplementationFactory;
		}
		if (Set.class.equals(propertyClass)) {
			return setImplementationFactory;
		}

		// primitive types
		if (propertyClass.isPrimitive()) {
			return new PrimitiveImplementationFactory(propertyClass);
		}
		
		// concrete java class
		if (!ClassUtils.isAbstract(propertyClass)) {
			return new DefaultImplementationFactory(propertyClass);
		}
		
		return null;
		
	}

	public void setListImplementationFactory(ImplementationFactory<?> listImplementationFactory) {
		this.listImplementationFactory = listImplementationFactory;
	}

	public void setMapImplementationFactory(ImplementationFactory<?> mapImplementationFactory) {
		this.mapImplementationFactory = mapImplementationFactory;
	}

	public void setSetImplementationFactory(ImplementationFactory<?> setImplementationFactory) {
		this.setImplementationFactory = setImplementationFactory;
	}

}
