/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.converter;

import java.lang.annotation.Annotation;
import java.math.BigDecimal;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Map;

import javax.inject.Singleton;

import org.joda.time.DateTime;
import org.joda.time.LocalDate;
import org.loom.binding.AnnotationContainer;
import org.loom.converter.date.DateConverter;
import org.loom.converter.date.DateTimeAdapterFormat;
import org.loom.converter.date.LocalDateAdapterFormat;
import org.loom.converter.date.SqlDateAdapterFormat;
import org.loom.converter.date.TemporalConverterFactory;
import org.loom.converter.date.TimeAdapterFormat;
import org.loom.converter.date.TimestampAdapterFormat;
import org.loom.converter.date.UtilDateAdapterFormat;
import org.loom.log.Log;
import org.loom.util.ClassUtils;


@Singleton
@SuppressWarnings("unchecked")
public class DefaultConverterFactory implements ConverterFactory {

	/** map of {@link AnnotationDrivenConverterFactory} factories */
	private Map<Class<Annotation>, AnnotationDrivenConverterFactory> annotationDrivenFactories = new HashMap<Class<Annotation>, AnnotationDrivenConverterFactory>();
	
	/** map of converters, using the property class as key */
	private Map<Class, Converter> convertersByClass = new HashMap<Class, Converter>();
	
	private static Log log = Log.getLog(DefaultConverterFactory.class);
	
	public DefaultConverterFactory() {
		this(true);
	}
	
	/**
	 * @param createDefaultConverters true to create the default converters
	 */
	public DefaultConverterFactory(boolean createDefaultConverters) {
		if (createDefaultConverters) {
			initDefaults();
		}
	}
	
	private void initDefaults() {
		
		// java types
		convertersByClass.put(BigDecimal.class, new BigDecimalConverter());
		convertersByClass.put(Boolean.class, new BooleanConverter());
		convertersByClass.put(Byte.class, new ByteConverter()); 
		convertersByClass.put(Character.class, new CharacterConverter());
		convertersByClass.put(Double.class, new DoubleConverter());
		//convertersByClass.put(Float.class, new FloatConverter()); Float will never be supported; use Double instead
		convertersByClass.put(Integer.class, new IntegerConverter());
		convertersByClass.put(Long.class, new LongConverter());
		convertersByClass.put(Short.class, new ShortConverter());
		convertersByClass.put(String.class, new StringConverter());
		
		// primitive types
		convertersByClass.put(Boolean.TYPE, new BooleanConverter().setNullValue(Boolean.FALSE));
		convertersByClass.put(Byte.TYPE, new ByteConverter().setNullValue((byte) 0)); 
		convertersByClass.put(Character.TYPE, new CharacterConverter().setNullValue('\0'));
		convertersByClass.put(Double.TYPE, new DoubleConverter().setNullValue(0d));
		//convertersByClass.put(Float.TYPE, new FloatConverter()); float will never be supported; use double instead
		convertersByClass.put(Integer.TYPE, new IntegerConverter().setNullValue(0));
		convertersByClass.put(Long.TYPE, new LongConverter().setNullValue(0L));
		convertersByClass.put(Short.TYPE, new ShortConverter().setNullValue((short) 0));
		
		// date types
		convertersByClass.put(java.util.Date.class, new DateConverter(java.util.Date.class, DateConverter.DATE_FORMAT_KEY, UtilDateAdapterFormat.class));
		convertersByClass.put(java.sql.Date.class, new DateConverter(java.sql.Date.class, DateConverter.DATE_FORMAT_KEY, SqlDateAdapterFormat.class));
		convertersByClass.put(java.sql.Timestamp.class, new DateConverter(Timestamp.class, DateConverter.DATE_TIME_FORMAT_KEY, TimestampAdapterFormat.class));
		convertersByClass.put(java.sql.Time.class, new DateConverter(Time.class, DateConverter.TIME_FORMAT_KEY, TimeAdapterFormat.class));
		if (ClassUtils.isPresent("org.joda.time.LocalDate")) {
			convertersByClass.put(ClassUtils.forName("org.joda.time.LocalDate"), new DateConverter(LocalDate.class, DateConverter.DATE_FORMAT_KEY, LocalDateAdapterFormat.class));
			convertersByClass.put(ClassUtils.forName("org.joda.time.DateTime"), new DateConverter(DateTime.class, DateConverter.DATE_TIME_FORMAT_KEY, DateTimeAdapterFormat.class));
		}
		
		// annotated types
		addConverterFactory(new ConverterMetadataConverterFactory());
		if (ClassUtils.isPresent("javax.persistence.Temporal")) {
			addConverterFactory(new TemporalConverterFactory());
		}
		
	}

	public void setDefaultConverterForClass(Class propertyClass, Converter converter) {
		convertersByClass.put(propertyClass, converter);
	}
	
	/**
	 * @return the default converter for simple classes
	 */
	public Converter getConverterForClass(Class propertyClass) {
		if (Enum.class.isAssignableFrom(propertyClass)) {
			return new EnumConverter(propertyClass);
		}
		return convertersByClass.get(propertyClass);
	}

	/**
	 * Usually the Converter is specified by the Class at the end of the chain or any annotation
	 * prior to that point that specifies what must be done with that property (a \@Converter 
	 * annotation with a configured propertyPath)
	 * @return the Converter for the leaf Node at the end of a Node chain 
	 */
	public Converter getConverter(AnnotationContainer wrapper) {
		
		// annotation-driven converter factories
		for (Annotation annotation : wrapper.getAnnotations()) {
			AnnotationDrivenConverterFactory factory = annotationDrivenFactories.get(annotation.annotationType());
			if (factory != null) {
				return factory.getConverter(wrapper, annotation);
			}
		}
		
		/*
		 * old  implementation, worse performance
		for (Map.Entry<Class<Annotation>, AnnotationDrivenConverterFactory> entry : annotationDrivenFactories.entrySet()) {
			Annotation annotation = wrapper.getAnnotation(entry.getKey());
			if (annotation != null) {
				return entry.getValue().getConverter(wrapper, annotation);
			}
		}*/
		
		// the default converter for simple property classes
		return getConverterForClass(wrapper.getPropertyClass());
	}
	
	public void setConverters(Map<Class, Converter> convertersByClass) {
		this.convertersByClass.putAll(convertersByClass);
	}
	
	public void setConverter(Class clazz, Converter converter) {
		log.debug("Adding converter: ", converter.getClass().getName());
		convertersByClass.put(clazz, converter);
	}

	public void addConverterFactory(AnnotationDrivenConverterFactory factory) {
		log.debug("Adding converter factory: ", factory.getClass().getName());
		annotationDrivenFactories.put(factory.getAnnotationClass(), factory);
	}
	
}
