package org.loom.guice;

import org.codehaus.jackson.map.annotate.JsonDeserialize;
import org.loom.action.Action;
import org.loom.annotation.processor.AnnotationProcessor;
import org.loom.converter.AnnotationDrivenConverterFactory;
import org.loom.converter.Converter;

import com.google.inject.Binder;
import com.google.inject.TypeLiteral;
import com.google.inject.multibindings.Multibinder;

/**
 * Handles all configured plugins with Loom 
 * @author icoloma
 *
 */
public final class Plugins {

	private Plugins() { }
	
	/**
	 * Bind a list of jackson mixins to specify the default serializer and deserializer for java classes.
	 * The provided Jackson mixins must specify a {@link JsonDeserialize#as()} value, which will be 
	 * used to bind the deserializer to the java class.
	 * Details about mixins in Jackson can be seen <a href="http://wiki.fasterxml.com/JacksonMixInAnnotations">here</a>
	 * Example of use: Plugins.bindJacksonMixins(binder(), GeoPtJacksonMixin.class); 
	 * @param binder the binder value
	 * @param mixins the list of mixins to apply
	 */
	public static void bindJacksonMixins(Binder binder, Iterable<Class> mixins) {
		Multibinder<Class> mixinsBinder = Multibinder.newSetBinder(binder, new TypeLiteral<Class>() {}, JacksonMixins.class);
		if (mixins != null) {
			for (Class mixin : mixins) {
				mixinsBinder.addBinding().toInstance(mixin);
			}
		}
	}
	
	/**
	 * Register a list of action classes in Loom
	 * Example of use: Plugins.bindActions(binder(), MortgagesAction.class, CustomersAction.class); 
	 * @param binder the binder value
	 * @param actionClasses the list of action classes to register
	 */
	public static void bindActions(Binder binder, Iterable<Class<? extends Action>> actionClasses) {
		// register actions set
		Multibinder<Class<Action>> actionsBinder = Multibinder.newSetBinder(binder, new TypeLiteral<Class<Action>>() {} );
		if (actionClasses != null) {
			for (Class actionClass : actionClasses) {
				actionsBinder.addBinding().toInstance(actionClass);
			}
		}

	}

	/**
	 * Register a list of converter factories
	 * Example of use: Plugins.bindActions(binder(), JsonConverterFactory.class); 
	 * @param binder the binder value
	 * @param factories the list of factory classes to register
	 */
	public static void bindConverterFactories(Binder binder, Iterable<Class<? extends AnnotationDrivenConverterFactory>> factories) {
		// register pluggable converters
		Multibinder<AnnotationDrivenConverterFactory> apBinder = Multibinder.newSetBinder(binder, AnnotationDrivenConverterFactory.class);
		if (factories != null) {
			for (Class<? extends AnnotationDrivenConverterFactory> factoryClass : factories) {
				apBinder.addBinding().to(factoryClass);
			}
		}
	}

	/**
	 * Register a list of converter classes
	 * Example of use: Plugins.bindActions(binder(), StringConverter.class, IntegerConverter.class); 
	 * @param binder the binder value
	 * @param converters the list of converter classes to register
	 */
	public static void bindConverters(Binder binder, Iterable<Class<? extends Converter>> converters) {
		Multibinder<Converter> convBinder = Multibinder.newSetBinder(binder, Converter.class);
		if (converters != null) {
			for (Class<? extends Converter> converterClass : converters) {
				convBinder.addBinding().to(converterClass);
			}
		}
		
	}

	public static void bindAnnotationProcessors(Binder binder, Iterable<Class<? extends AnnotationProcessor>> annotationProcessorClasses) {
		Multibinder<AnnotationProcessor> apBinder = Multibinder.newSetBinder(binder, AnnotationProcessor.class);
		if (annotationProcessorClasses != null) {
			for (Class<? extends AnnotationProcessor> annotationProcessorClass : annotationProcessorClasses) {
				apBinder.addBinding().to(annotationProcessorClass); 
			}
		}
	}
	
}
