/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.i18n;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * A message for the end user. Messages are translated according to message templates 
 * retrieved from a MessageRepository.
 * Each message includes all the arguments needed for the translation, indicating whether
 * it should be translated or not.
 * @author icoloma
 */
public class Message implements Externalizable {

	/** the property path */
	public static final String PROPERTY_PATH_ARG = "propertyName";
	
	/** the value */
	public static final String VALUE_ARG = "value";

	/** the message priority level, defaults to ERROR (immutable, set at constructor) */ 
	private MessageLevel level;
	
	/** the message key in the {@link MessagesRepository} (immutable, set at constructor) */
	private String messageKey;
	
	/** the property name, if this message is bound to a specific property. Can be null (immutable, set at constructor) */
	private String propertyPath;
	
	/** not null if this message was produced in a previous request and serialized. In this case, its contents is the serialized value */
	private String serializedValue;
	
	/** the map of arguments that will be applied 'as is' */
	private Map<String, Object> args = new HashMap<String, Object>();
	
	/** the map of arguments that will be translated */
	private Map<String, Object> translatedArgs = new HashMap<String, Object>();

	/** 
	 * This is used to keep track of which messages have NOT been printed in the HTML output, 
	 * which will be logged as an error. This is done to guarantee that no validation error 
	 * gets unnoticed to the user.
	 */
	private boolean rendered;

	public Message() {
		// for class deserialization only. DO NOT USE!
	}
	
	public Message(String messageKey) {
		this(MessageLevel.ERROR, messageKey, null);
	}
	
	public Message(String messageKey, String propertyPath) {
		this(MessageLevel.ERROR, messageKey, propertyPath);
	}

	public Message(MessageLevel level, String messageKey) {
		this(level, messageKey, null);
	}
	
	public Message(MessageLevel level, String messageKey, String propertyPath) {
		this.level = level;
		this.messageKey = messageKey;
		this.propertyPath = propertyPath;
	}
	
	@Override
	public String toString() {
		return new ToStringBuilder(this)
			.append("level", level)
			.append("messageKey", messageKey)
			.append("propertyPath", propertyPath)
			.toString();
	}

	public Map<String, Object> getArgs() {
		return args;
	}

	public String getMessageKey() {
		return messageKey;
	}

	public Map<String, Object> getTranslatedArgs() {
		return translatedArgs;
	}

	public String getPropertyPath() {
		return propertyPath;
	}

	public Message addArg( String arg, Object value){
		getArgs().put(arg,value);
		return this;
	}
	
	public Message addTranslatedArg( String arg, Object value ){
		getTranslatedArgs().put(arg,value);
		return this;
	}

	public boolean isRendered() {
		return rendered;
	}

	public void setRendered(boolean rendered) {
		this.rendered = rendered;
	}

	public MessageLevel getLevel() {
		return level;
	}

	public boolean isDeserialized() {
		return serializedValue != null;
	}

	public void readExternal(ObjectInput in) throws IOException,
			ClassNotFoundException {
		level = (MessageLevel) in.readObject();
		propertyPath = (String) in.readObject();
		messageKey = (String) in.readObject();
		serializedValue = (String) in.readObject();
	}

	public void writeExternal(ObjectOutput out) throws IOException {
		out.writeObject(level);
		out.writeObject(propertyPath);
		out.writeObject(messageKey);
		out.writeObject(serializedValue);
	}

	public String getSerializedValue() {
		return serializedValue;
	}

	public void setSerializedValue(String serializedValue) {
		this.serializedValue = serializedValue;
	}

}
