/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.i18n;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.loom.log.Log;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.Scope;
import org.loom.servlet.names.FlashAttributeNames;

import com.google.common.base.Predicate;
import com.google.common.collect.Collections2;


/**
 * The collection of messages to the user. This class puts together any error,
 * warning or informational message for the user. Note that messages can be bound
 * to a concrete property or not.
 * Messages are shown in the same order as they are produced, which usually is the same 
 * order in which validations are defined in the annotations. 
 * @author Ignacio Coloma
 */
public class Messages implements Serializable {

	/** the list of messages bound to properties */
	private Map<String, Message> boundMessages = new LinkedHashMap<String, Message>();
	
	/** the list of messages not bound to any property*/
	private List<Message> unboundMessages = new ArrayList<Message>();
	
	private static Log log = Log.getLog(Messages.class);
	
	/**
	 * Adds a new Message
	 */
	public Message addMessage(Message message) {
		String propertyName = message.getPropertyPath();
		if (propertyName == null) {
			unboundMessages.add(message);
		} else {
			if (message.getLevel() != MessageLevel.ERROR) {
				throw new IllegalArgumentException("Only ERROR bound messages are supported. Please use unbound instead");
			}
			
			// do not overwrite any existing error message. This prevents the following scenarios
			// validation erors overwriting conversion errors
			// conversion errors overwriting serialized conversion errors, which would interrupt the normal 
			// workflow where it shouldn't (after a error redirection)
			if (!boundMessages.containsKey(propertyName)) {
				boundMessages.put(propertyName, message);
			}
		}
		return message;
	}
	
    /**
     * Add all messages from another Messages instance
     */
    public Messages addAll(Messages messages) {
    	for (Message m : messages.getAllMessages()) {
    		addMessage(m);
    	}
    	return this;
    }
    
	public Message error(String propertyPath, String messageKey) {
		return addMessage(new Message(MessageLevel.ERROR, messageKey, propertyPath));
	}
	
	public Message error(String messageKey) {
		return addMessage(new Message(MessageLevel.ERROR, messageKey));
	}
	
	public Message info(String messageKey) {
		return addMessage(new Message(MessageLevel.INFO, messageKey));
	}
	
	public Message warn(String messageKey) {
		return addMessage(new Message(MessageLevel.WARN, messageKey));
	}
	
	/**
	 * Adds a message
	 * @param propertyPath the non-normalized property path. Can be null.
	 * @deprecated use error(propertyPath, messageKey) instead
	 */
	public Message addMessage(String propertyPath, String messageKey) {
		return error(propertyPath, messageKey);
	}
	
	/**
	 * Add a unbound message
	 * @deprecated use error(messageKey) instead
	 */
	public Message addMessage(String messageKey) {
		return addMessage(new Message(messageKey));
	}
    
	/**
	 * @return the Message for that property, null if none
	 */
	public Message get(String propertyName) {
		return boundMessages.get(propertyName);
	}
	
	public boolean isEmpty() {
		return boundMessages.isEmpty() && unboundMessages.isEmpty();
	}
	
	/**
	 * @return the total number of messages of any priority
	 */
	public int size() {
		return boundMessages.size() + unboundMessages.size();
	}
	
	public Collection<Message> getBoundMessages() {
		return boundMessages.values();
	}
	
	public Collection<Message> getBoundMessages(MessageLevel level) {
		return Collections2.filter(boundMessages.values(), new MessageSelectorPredicate(level));
	}

	public Collection<Message> getUnboundMessages() {
		return unboundMessages;
	}
	
	public Collection<Message> getUnboundMessages(MessageLevel level) {
		return Collections2.filter(unboundMessages, new MessageSelectorPredicate(level));
	}
	
	/**
	 * @return a list with all bound and unbound messages 
	 */
	public List<Message> getAllMessages() {
		List<Message> result = new ArrayList<Message>();
		result.addAll(unboundMessages);
		result.addAll(boundMessages.values());
		return result;
	}

	/**
	 * @param name the name of the input tag (the property path)
	 * @return true if the component has any associated error
	 */
	public boolean hasAnyError(String name) {
		return get(name) != null; 
	}
	
	@Override
	public String toString() {
		return new ToStringBuilder(this)
			.append("boundMessages (" + boundMessages.size() + ")", boundMessages.values())
			.append("unboundMessages (" + unboundMessages.size() + ")", unboundMessages)
			.toString()
			;
	}

	private void logUnrenderedMessages(Iterable<Message> list, MessagesRepository repository) {
		for (Message m : list) {
			if (!m.isRendered()) {
				log.warn("The following message has NOT been included in the HTML output (check your jsp page): [", m.getLevel(), "] ", repository.translateMessage(m));
			}
		}
	}
	
	/**
	 * If there are messages that have not been included in the HTML response, log about this as an error.
	 * @param repository the messagesRepository used to translate the messages before logging.
	 */
	public void logUnrenderedMessages(MessagesRepository repository) {
		logUnrenderedMessages(unboundMessages, repository);
		logUnrenderedMessages(boundMessages.values(), repository);
	}

	/**
	 * Saves this {@link Messages} instance to the flash context for a redirect. 
	 * If this instance is empty, does nothing.
	 */
	public void saveForRedirect(LoomServletRequest request) {
		if (!this.isEmpty()) {
			MessagesRepository mr = request.getMessagesRepository();
			for (Message message : getAllMessages()) {
				message.setSerializedValue(mr.translateMessage(message));
			}
			request.setScopedAttribute(FlashAttributeNames.SAVED_MESSAGES, this, Scope.FLASH);
		}
	}
	
	/**
	 * restore the {@link Messages} instance from flash context after a redirect. 
	 */
	public void restoreFromRedirect(LoomServletRequest request) {
		Messages restoredMessages = (Messages) request.getScopedAttribute(FlashAttributeNames.SAVED_MESSAGES, Scope.FLASH);
		if (restoredMessages != null) {
			for (Message m : restoredMessages.getAllMessages()) {
	    		addMessage(m);
	    	}
		}
	}
	
	private static class MessageSelectorPredicate implements Predicate<Message> {
		
		private MessageLevel level;
		
		private MessageSelectorPredicate(MessageLevel level) {
			this.level = level;
		}

		public boolean apply(Message message) {
			return level.equals(message.getLevel());
		}
		
	}
	
}
