package org.loom.i18n;

import java.io.IOException;
import java.util.Locale;
import java.util.Set;
import java.util.concurrent.ConcurrentMap;

import javax.inject.Inject;

import org.loom.config.Delegate;
import org.springframework.core.io.Resource;

import com.google.common.collect.Maps;

/**
 * MessagesRepository instance used while in development.
 * This instance will check if any resource has been modified and reload the 
 * contents if needed.
 * @author icoloma
 *
 */
public class ReloadableMessagesRepositoryFactory implements MessagesRepositoryFactory {

	private MessagesRepositoryFactory delegate;
	
	private ConcurrentMap<Resource, Long> resourcesTimestamps = Maps.newConcurrentMap();
	
	@Override
	public MessagesRepository getRepository(Locale locale) {
		MessagesRepository repository = delegate.getRepository(locale);
		boolean reload = false;
		for (Resource res : repository.getResources()) {
			try {
				Long lastKnownTimestamp = resourcesTimestamps.get(res);
				if (lastKnownTimestamp == null) {
					resourcesTimestamps.put(res, res.getFile().lastModified());
				} else if (lastKnownTimestamp != -1L){
					long current = res.getFile().lastModified();
					if (current != lastKnownTimestamp) {
						resourcesTimestamps.put(res, current);
						reload = true;
					}
				}
			} catch (IOException e) {
				// the resource cannot be read, ignore
				resourcesTimestamps.put(res, -1L);
			}
		}
		if (reload) {
			repository.initialize();
		}
		return repository;
	}

	@Override
	public void addBrowserMessage(String messageKey) {
		delegate.addBrowserMessage(messageKey);
	}

	@Override
	public void setBrowserMessages(Set<String> browserMessages) {
		delegate.setBrowserMessages(browserMessages);
	}

	@Override
	public Locale getDefaultLocale() {
		return delegate.getDefaultLocale();
	}

	public MessagesRepositoryFactory getDelegate() {
		return delegate;
	}

	@Inject 
	public void setDelegate(@Delegate MessagesRepositoryFactory delegate) {
		this.delegate = delegate;
	}

}
