package org.loom.interceptor;

import java.io.Serializable;

import javax.persistence.NoResultException;

import org.loom.converter.Converter;
import org.loom.exception.EntityNotFoundException;
import org.loom.exception.OptimisticLockException;
import org.loom.i18n.Messages;
import org.loom.log.Log;
import org.loom.mapping.Event;
import org.loom.mapping.ParsedAction;
import org.loom.persistence.ExtendedEntityManager;
import org.loom.resolution.Resolution;
import org.loom.servlet.params.ServletRequestParameters;
import org.loom.util.PropertyUtils;

public class DefaultRetrieveEntityInterceptor extends AbstractPropertyBoundInterceptor implements RetrieveEntityInterceptor, PopulateInterceptor {

	/** the propertyBinder of the persisten object version */
	private String versionPropertyPath;

	/** true to mark the instance as readOnly after loading */
	private boolean readOnly;

	/** the full path of the ID property, default to the entity PK name */
	private String idPropertyPath;
	
	/** the id property name */
	private String idPropertyName;
	
	/** the property class */
	private Class<?> propertyClass;
	
	/** boolean if the idPropertyPath corresponds with the primary key */
	private boolean usingPrimaryKey;
	
	private ExtendedEntityManager entityManager;
	
	private static Log log = Log .getLog(DefaultRetrieveEntityInterceptor.class);

	public Resolution beforePopulate(ParsedAction action) {
		ServletRequestParameters params = action.getRequest().getRequestParameters();
		Object retrievedEntity = null;
		String idValueAsString = params.getStringParameter(idPropertyPath);
		Event event = action.getEvent();
		if (idValueAsString != null) {
			Messages messages = action.getMessages();
			Converter converter = event.getConverter(idPropertyPath);
			if (converter == null) {
				throw new IllegalArgumentException("No converter found for property " + action.getAction().getClass().getSimpleName() + "." + idPropertyPath);
			}
			Serializable idValueAsObject = (Serializable) converter.getAsObject(idPropertyPath, idValueAsString, messages, action.getRequest().getMessagesRepository());
			if (messages.get(idPropertyPath) == null) { // conversion ok
				//PropertyBinder propertyBinder = getPropertyBinder();
				if (idValueAsObject != null) {
					try {
						retrievedEntity = usingPrimaryKey? entityManager.find(propertyClass, idValueAsObject) :
							entityManager.findSingle("from " + propertyClass.getSimpleName() + " i where i." + idPropertyName  + "=?", idValueAsObject);
						if (readOnly) {
							entityManager.setReadOnly(retrievedEntity, readOnly);
						}
					} catch (NoResultException e) {
						throw new EntityNotFoundException(propertyClass, idValueAsObject, e);
					}
				} else {
					retrievedEntity = null;
				}
				// remove the param so it does not get assigned again on
				// populate time
				params.setAssigned(idPropertyPath);
				log.debug("Assigning value ", getPropertyPath(), " = ", retrievedEntity);
				action.setPropertyAsObject(propertyPath, retrievedEntity);
				checkVersionNumber(action, retrievedEntity);
			}
		}
		return null;
	}

	/**
	 * Check if the entity can be updated based on the configured \@Version property
	 */
	private void checkVersionNumber(ParsedAction action, Object retrievedEntity) {
		if (versionPropertyPath != null) {
			String versionValueAsString = action.getRequest().getStringParameter(versionPropertyPath);
			if (versionValueAsString != null) {
				Object dbVersion = action.getPropertyAsObject(versionPropertyPath);
				Object newVersion = action.getEvent().getConverter(versionPropertyPath).getAsObject(versionPropertyPath, versionValueAsString, action.getMessages(), action.getRequest().getMessagesRepository());

				if (dbVersion == null || dbVersion.equals(0)) {
					// new version should be null or zero
					if (!(newVersion == null || newVersion.equals(0))) {
						throw new OptimisticLockException(retrievedEntity.getClass(), dbVersion, newVersion);
					}
				} else {
					// existing instance, just check version numbers
					if (!dbVersion.equals(newVersion)) {
						throw new OptimisticLockException(retrievedEntity.getClass(), dbVersion, newVersion);
					}
				}
			}
		}
	}

	public void setReadOnly(boolean readOnly) {
		this.readOnly = readOnly;
	}

	@Override
	public String toString() {
		return "RetrieveEntityInterceptor { property=" + getPropertyPath() + ", readOnly = " + readOnly + "}";
	}

	public void setEntityManager(ExtendedEntityManager entityManager) {
		this.entityManager = entityManager;
	}

	public String getIdPropertyPath() {
		return idPropertyPath;
	}

	public void setIdPropertyName(String idPropertyName) {
		this.idPropertyName = idPropertyName;
		this.idPropertyPath = PropertyUtils.concatPropertyPath(getPropertyPath(), idPropertyName);
		this.usingPrimaryKey = false;
	}

	public void setPropertyClass(Class<?> propertyClass) {
		this.propertyClass = propertyClass;
		if (idPropertyPath == null) {
			setIdPropertyName(entityManager.getIdPropertyName(propertyClass));
			this.usingPrimaryKey = true;
		}
		String versionName = entityManager.getVersionPropertyName(propertyClass);
		if (versionName != null) {
			this.versionPropertyPath = PropertyUtils.concatPropertyPath(getPropertyPath(), versionName);
		}
	}
}
