/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.interceptor;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.loom.mapping.ParsedAction;
import org.loom.resolution.Resolution;
import org.loom.tags.Button;
import org.loom.tags.Form;
import org.loom.tags.Link;
import org.loom.validator.ButtonDecorator;
import org.loom.validator.FormDecorator;
import org.loom.validator.LinkDecorator;

/**
 * Maintains a list of listeners.
 * For performance, this listener maintains a separate list for each phase of 
 * the event execution.
 * @author Ignacio Coloma
 */
@SuppressWarnings("unchecked")
public class InterceptorProxy implements CreateInterceptor, PopulateInterceptor, ValidateInterceptor, 
	ExecuteInterceptor, FinallyListener, ResolutionInterceptor, ErrorListener, LinkDecorator, FormDecorator, ButtonDecorator {

	/** listeners associated to each event */
	private List<CreateInterceptor> createListeners = new ArrayList<CreateInterceptor>();
	private List<PopulateInterceptor> populateListeners = new ArrayList<PopulateInterceptor>();
	private List<ValidateInterceptor> validateListeners = new ArrayList<ValidateInterceptor>();
	private List<ExecuteInterceptor> executeListeners = new ArrayList<ExecuteInterceptor>();
	private List<ResolutionInterceptor> resolutionListeners = new ArrayList<ResolutionInterceptor>();
	private List<FinallyListener> finallyListeners = new ArrayList<FinallyListener>();
	private List<ErrorListener> errorListeners = new ArrayList<ErrorListener>();
	
	/** interceptors that modify any link pointing to this event */
	private List<LinkDecorator> linkDecorators = new ArrayList<LinkDecorator>();
	
	/** interceptors that modify any form pointing to this event */
	private List<FormDecorator> formDecorators = new ArrayList<FormDecorator>();
	
	/** interceptors that modify any button pointing to this event */
	private List<ButtonDecorator> buttonDecorators = new ArrayList<ButtonDecorator>();

	/** list of all listeners registered */
	private Set<Interceptor> allListeners = new HashSet<Interceptor>();
	
	/**
	 * Adds a listener that implements one or more of the listener interfaces
	 */
	public <T extends Interceptor> void addListener(T listener) {
		allListeners.add(listener);
		if (listener instanceof CreateInterceptor) {
			createListeners.add((CreateInterceptor) listener);
		}
		if (listener instanceof PopulateInterceptor) {
			populateListeners.add((PopulateInterceptor) listener);
		}
		if (listener instanceof ValidateInterceptor) {
			validateListeners.add((ValidateInterceptor) listener);
		}
		if (listener instanceof ExecuteInterceptor) {
			executeListeners.add((ExecuteInterceptor) listener);
		}
		if (listener instanceof ResolutionInterceptor) {
			resolutionListeners.add((ResolutionInterceptor) listener);
		}
		if (listener instanceof FinallyListener) {
			finallyListeners.add((FinallyListener) listener);
		}
		if (listener instanceof LinkDecorator) {
			linkDecorators.add((LinkDecorator) listener);
		}
		if (listener instanceof FormDecorator) {
			formDecorators.add((FormDecorator) listener);
		}
		if (listener instanceof ButtonDecorator) {
			buttonDecorators.add((ButtonDecorator) listener);
		}
		if (listener instanceof ErrorListener) {
			errorListeners.add((ErrorListener) listener);
		}
	}
	
	/**
	 * @return a Set of all registered interceptors of the supplied class
	 */
	public <T extends Interceptor> Set<T> findInterceptorsWithClass(Class<T> interceptorClass) {
		Set<T> result = new HashSet<T>();
		for (Interceptor interceptor : allListeners) {
			if (interceptorClass.isAssignableFrom(interceptor.getClass())) {
				result.add((T)interceptor);
			}
		}
		return result;
	}
	
	/**
	 * Remove the supplied interceptor instance from this list
	 * @param listener the instance to be removed
	 */
	public void remove(Interceptor listener) {
		allListeners.remove(listener);
		if (listener instanceof CreateInterceptor) {
			createListeners.remove(listener);
		}
		if (listener instanceof PopulateInterceptor) {
			populateListeners.remove(listener);
		}
		if (listener instanceof ValidateInterceptor) {
			validateListeners.remove(listener);
		}
		if (listener instanceof ExecuteInterceptor) {
			executeListeners.remove(listener);
		}
		if (listener instanceof ResolutionInterceptor) {
			resolutionListeners.remove(listener);
		}
		if (listener instanceof FinallyListener) {
			finallyListeners.remove(listener);
		}
		if (listener instanceof LinkDecorator) {
			linkDecorators.remove(listener);
		}
		if (listener instanceof ErrorListener) {
			errorListeners.remove(listener);
		}
	}

	
	public Resolution afterCreate(ParsedAction action) {
		for (CreateInterceptor listener : createListeners) {
			Resolution result = listener.afterCreate(action);
			if (result != null) {
				return result;
			}
		}
		return null;
	}

	public Resolution beforePopulate(ParsedAction action) {
		for (PopulateInterceptor listener : populateListeners) {
			Resolution result = listener.beforePopulate(action);
			if (result != null) {
				return result;
			}
		}
		return null;
	}

	public Resolution beforeValidate(ParsedAction action) {
		for (ValidateInterceptor listener : validateListeners) {
			Resolution result = listener.beforeValidate(action);
			if (result != null) {
				return result;
			}
		}
		return null;
	}

	public Resolution beforeExecute(ParsedAction action) {
		for (ExecuteInterceptor listener : executeListeners) {
			Resolution result = listener.beforeExecute(action);
			if (result != null) {
				return result;
			}
		}
		return null;	
	}
	
	public void beforeResolution(ParsedAction action, Resolution resolution) {
		for (ResolutionInterceptor listener : resolutionListeners) {
			listener.beforeResolution(action, resolution);
		}
	}
	
	public Resolution onError(ParsedAction action) {
		for (ErrorListener listener : errorListeners) {
			Resolution result = listener.onError(action);
			if (result != null) {
				return result;
			}
		}
		return null;
	}
	
	public void doFinally(ParsedAction action, Exception e) {
		for (FinallyListener listener : finallyListeners) {
			listener.doFinally(action, e);
		}
	}
	
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("{");
		appendListenerList(builder, "createListeners", createListeners);
		appendListenerList(builder, "populateListeners", populateListeners);
		appendListenerList(builder, "validateListeners", validateListeners);
		appendListenerList(builder, "executeListeners", executeListeners);
		appendListenerList(builder, "resolutionListeners", resolutionListeners);
		appendListenerList(builder, "finallyListeners", finallyListeners);
		appendListenerList(builder, "errorListeners", errorListeners);
		builder.append("}");
		return builder.toString();
	}
	
	private void appendListenerList(StringBuilder b, String name, List<?> l) {
		if (!l.isEmpty()) {
			b.append(name).append(" = { ");
			b.append(StringUtils.join(l.toArray(), ", "));
			b.append("}, ");
		}
	}
	
	public boolean isEmpty() {
		return allListeners.isEmpty();
	}

	public void decorate(Link link) {
		for (LinkDecorator decorator : linkDecorators) {
			decorator.decorate(link);
		}
	}
	
	public void decorate(Form form) {
		for (FormDecorator decorator : formDecorators) {
			decorator.decorate(form);
		}
	}

	public void decorate(Button button) {
		for (ButtonDecorator decorator : buttonDecorators) {
			decorator.decorate(button);
		}
	}
	
	public List<CreateInterceptor> getCreateListeners() {
		return createListeners;
	}

	public List<PopulateInterceptor> getPopulateListeners() {
		return populateListeners;
	}

	public List<ValidateInterceptor> getValidateListeners() {
		return validateListeners;
	}

	public List<ExecuteInterceptor> getExecuteListeners() {
		return executeListeners;
	}

	public List<FinallyListener> getFinallyListeners() {
		return finallyListeners;
	}

	public List<ErrorListener> getErrorListeners() {
		return errorListeners;
	}

}
