/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.log;

import java.lang.reflect.Array;
import java.util.Iterator;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A logger class.
 * Log arguments will not be concatenated unless necessary, according to the configured log level.
 * A static factory method has been added that guesses the Logging class using the current stack trace. 
 * 
 * The Log class should be instantiated using a static private attribute:
 * 
 * <code>
 * public class MyClass {
 * 
 * private static Log log = Log.getLog();
 * 
 * public void method() {
 * ...
 * log.debug(this.getFoo(), " will not be serialized unless needed");
 * ...
 * }
 * 
 * }
 * </code>
 * 
 * Notice the use of commas instead of '+'.
 * 
 * @author icoloma
 */
public class Log {

	private Logger delegate;

	private Log(Logger delegate) {
		this.delegate = delegate;
	}

	/**
	 * @deprecated use getLog(Class) instead. This class instantiates an Exception to calculate the stack trace, which consumes resources on deployment time.
	 * @return a Logger configured for the calling class (the class that called this method)
	 */
	public static Log getLog() {
		StackTraceElement[] stackTrace = Thread.currentThread().getStackTrace();
		String logName = "unspecified"; 
		for (int i = 2; i < stackTrace.length; i++) {
			String stackNode = stackTrace[i].getClassName();
			// avoid java.lang.Thread and org.loom.Log, and their position in the stack trace may vary
			if (!stackNode.startsWith("java.") && !stackNode.startsWith("org.loom.log")) {
				logName = stackNode;
				break;
			}
		}
		return new Log(LoggerFactory.getLogger(logName));
	}
	
	/**
	 * @return a Logger configured for the specified class (this method should be rarely used)
	 */
	public static Log getLog(Class<?> clazz) {
		return new Log(LoggerFactory.getLogger(clazz));
	}
	
	public boolean isTraceEnabled() {
		return delegate.isTraceEnabled();
	}

	public boolean isDebugEnabled() {
		return delegate.isDebugEnabled();
	}
	
	public boolean isInfoEnabled() {
		return delegate.isInfoEnabled();
	}
	
	public boolean isWarnEnabled() {
		return delegate.isWarnEnabled();
	}
	
	public boolean isErrorEnabled() {
		return delegate.isErrorEnabled();
	}
	
	public final void fatal(Object... messageParts) {
		if (delegate.isErrorEnabled()) {
			delegate.error(concat(messageParts));
		}
	}

	public final void error(Object... messageParts) {
		if (delegate.isErrorEnabled()) {
			delegate.error(concat(messageParts));
		}
	}

	public final void warn(Object... messageParts) {
		if (delegate.isWarnEnabled()) {
			delegate.warn(concat(messageParts));
		}
	}

	public final void info(Object... messageParts) {
		if (delegate.isInfoEnabled()) {
			delegate.info(concat(messageParts));
		}
	}

	public final void debug(Object... messageParts) {
		if (delegate.isDebugEnabled()) {
			delegate.debug(concat(messageParts));
		}
	}

	public final void trace(Object... messageParts) {
		if (delegate.isTraceEnabled()) {
			delegate.trace(concat(messageParts));
		}
	}

	private String concat(Object[] parts) {
		StringBuilder builder = new StringBuilder();
		for (Object object : parts) {
			if (object != null && object.getClass().isArray()) {
				concatArray(builder, object);
			} else if (object instanceof Iterable<?>) {
				concatIterable(builder, (Iterable<?>) object);
			} else {
				builder.append(object);
			}
		}
		return builder.toString();
	}

	private void concatIterable(StringBuilder builder, Iterable<?> iterable) {
		builder.append("[ ");
		for (Iterator<?> i = iterable.iterator(); i.hasNext(); ) {
			builder.append(i.next());
			if (i.hasNext()) {
				builder.append(", ");
			}
		}
		builder.append(" ]");
	}
	
	private void concatArray(StringBuilder builder, Object object) {
		builder.append("[ ");
		int length = Array.getLength(object);
		for (int i = 0; i < length; i++) {
			builder.append(Array.get(object, i));
			if (i < length - 1) {
				builder.append(", ");
			}
		}
		builder.append(" ]");
	}

	/**
	 * @deprecated use error() instead
	 * @param throwable
	 * @param messageParts
	 */
	public final void fatal(Throwable throwable, Object... messageParts) {
		if (delegate.isErrorEnabled()) {
			delegate.error(concat(messageParts), throwable);
		}
	}

	public final void error(Throwable throwable, Object... messageParts) {
		if (delegate.isErrorEnabled()) {
			delegate.error(concat(messageParts), throwable);
		}
	}

	public final void warn(Throwable throwable, Object... messageParts) {
		if (delegate.isWarnEnabled()) {
			delegate.warn(concat(messageParts), throwable);
		}
	}

	public final void info(Throwable throwable, Object... messageParts) {
		if (delegate.isInfoEnabled()) {
			delegate.info(concat(messageParts), throwable);
		}
	}

	public final void debug(Throwable throwable, Object... messageParts) {
		if (delegate.isDebugEnabled()) {
			delegate.debug(concat(messageParts), throwable);
		}
	}

	public final void trace(Throwable throwable, Object... messageParts) {
		if (delegate.isTraceEnabled()) {
			delegate.trace(concat(messageParts), throwable);
		}
	}

	public Logger getDelegate() {
		return delegate;
	}

}
