/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.mapping;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;

import javax.inject.Inject;
import javax.inject.Singleton;

import org.loom.action.Action;
import org.loom.config.Delegate;
import org.loom.log.Log;
import org.loom.servlet.LoomServletRequest;
import org.loom.util.ClassUtils;
import org.zeroturnaround.javarebel.ClassEventListener;
import org.zeroturnaround.javarebel.Reloader;
import org.zeroturnaround.javarebel.ReloaderFactory;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.SetMultimap;

/**
 * ActionMappingRepository that detects changes to classes in the background.
 * To be used with JavaRebel.
 * 
 * The current implementation supports modifications to any Action class or dependent classes
 * (classes with annotations that have created validators in any action).  
 * Class additions or deletions are not yet supported. 
 * 
 * @author icoloma
 */
@Singleton
public class ReloadableActionMappingRepository implements ActionMappingRepository, ClassEventListener {

	/** map of mappings affected by each class reload */
	private SetMultimap<Class<?>, ActionMapping> affectedMappings = HashMultimap.create();
	
	/** true to catch any exception that may happen during class reloading, default true */
	private boolean catchReloadingExceptions = true;
	
	/** the real {@link ActionMappingRepository} instance */
	private ActionMappingRepository delegate;

	@Inject
	private ActionMappingFactory actionMappingFactory;
	
	public ReloadableActionMappingRepository(ActionMappingRepository delegate) {
		this.delegate = delegate;
	}
	
	private static Log log = Log
			.getLog(ReloadableActionMappingRepository.class);
	
	public ReloadableActionMappingRepository() {
		Reloader reloaderFactory = ReloaderFactory.getInstance();
		//reloaderFactory.addClassLoadListener(this);
		reloaderFactory.addClassReloadListener(this);
	}
	
	public int priority() {
		return PRIORITY_WEB;
	}
	
	/**
	 * Load/Reload an action mapping
	 */
	@SuppressWarnings("unchecked")
	public void onClassEvent(int eventType, Class clazz) {
		// we are not interested in EVENT_LOADED, that would be triggered when first scanning 
		// for actions (as with any invocation to Class.forName()) and hurts performance A LOT.
		if (eventType == ClassEventListener.EVENT_RELOADED) {
			try {
				//propertyWrapperFactory.cleanCache(clazz);
				if (isAction(clazz)) {
					// new action added or modified
					reload(clazz);
				} else {
					// check mappings that could depend on this action (for example, a JPA-annotated class)
					for (ActionMapping mapping : new HashSet<ActionMapping>(affectedMappings.get(clazz))) {
						reload(mapping.getActionClass());
					}
				}
			} catch (RuntimeException e) {
				if (catchReloadingExceptions) {
					log.error(e, "Could not reload actions affected by class " + clazz.getName() + ": " + e.getMessage());
				} else {
					throw e;
				}
			} 
		}
	}
	
	/**
	 * @return true if it's a valid Action class, false otherwise
	 */
	public static boolean isAction(Class<?> clazz) {
		if (!Action.class.isAssignableFrom(clazz)) { // does not implement Action
			return false;
		}
		if (ClassUtils.isAbstract(clazz)) {
			log.debug("Action class ignored for being abstract: " + clazz.getName());
			return false;
		}
		return true;
	}
	
	/** 
	 * Reload the provided ActionMapping.
	 * THIS OPERATION IS NOT THREAD SAFE. Read operations while reloading can result in undefined behavior.
	 * @param clazz
	 */
	private void reload(Class<? extends Action> clazz) {
		log.info("Reloading ", clazz.getName());
		removeActionMapping(clazz);
		addActionMapping(actionMappingFactory.create(clazz));
	}
	
	@Override
	public Event getEvent(String uri, LoomServletRequest request) {
		Event event = delegate.getEvent(uri,  request);
		if (event != null) {
			ActionMapping mapping = event.getActionMapping();
			if (mustRefresh(mapping)) { // has been modified and re-registered, retrieve its info again
				return delegate.getEvent(uri, request);
			}
		}
		return event;
	}
	
	@Override
	public ActionMapping getActionMappingByName(String actionClassName) {
		ActionMapping mapping = delegate.getActionMappingByName(actionClassName);
		if (mustRefresh(mapping)) { // has been modified and re-registered, retrieve its info again
			return delegate.getActionMappingByName(actionClassName);
		}
		return mapping;
	}
	
	/**
	 * Returns true if an action class or any of its dependant classes has been modified 
	 * and must be reloaded.
	 * If any dependent class has been modified the entire mapping will be re-registered 
	 * again, so it can be retrieved again safely.
	 */
	private boolean mustRefresh(ActionMapping mapping) {
		Reloader reloader = ReloaderFactory.getInstance();
		boolean result = false;
		for (Class<?> clazz : mapping.getReloadingClasses()) {
			if (reloader.isReloadableClass(clazz)) {
				log.trace("Checking reload candidate: ", clazz.getName());
				result = result || reloader.checkAndReload(clazz); 
			}
		}
		return result;
	}
	
	@Override
	public ActionMapping removeActionMapping(Class<? extends Action> actionClass) {
		ActionMapping mapping = delegate.removeActionMapping(actionClass);
		if (mapping != null) {
			for (Iterator<Map.Entry<Class<?>, ActionMapping>> i = affectedMappings.entries().iterator(); i.hasNext();) {
				if (i.next().getValue() == mapping) {
					i.remove();
				}
			}
		}
		return mapping;
	}

	@Override
	public void addActionMapping(ActionMapping mapping) {
		delegate.addActionMapping(mapping);
		
		// add the classes we should listen to 
		for (Class<?> clazz : mapping.getReloadingClasses()) {
			affectedMappings.put(clazz, mapping);
		}
	}

	public void setCatchReloadingExceptions(boolean catchReloadingExceptions) {
		this.catchReloadingExceptions = catchReloadingExceptions;
	}
	
	@Override
	public Collection<String> getMappings() {
		return delegate.getMappings();
	}

	@Override
	public String getHandle(String uri) {
		return delegate.getHandle(uri);
	}

	@Override
	public ActionMapping getActionMappingByClass(
			Class<? extends Action> actionClass) {
		return delegate.getActionMappingByClass(actionClass);
	}

	@Override
	public void validateActionMappings() {
		delegate.validateActionMappings();
	}

	public void setActionMappingFactory(ActionMappingFactory actionMappingFactory) {
		this.actionMappingFactory = actionMappingFactory;
	}

	@Inject 
	public void setDelegate(@Delegate ActionMappingRepository delegate) {
		this.delegate = delegate;
	}

}
