/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.resolution;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.codehaus.jackson.map.ObjectMapper;
import org.loom.servlet.CacheControl;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;

/**
 * A resolution that uses Jackson to serialize a JSON response.
 * @author Ignacio Coloma
 */
public class JacksonResolution extends AbstractHttpResolution {

	/** the object that does the real marshalling */
	private ObjectMapper mapper;
	
	/** the contents of this response object */
	private Object contents;
	
	/** if specified, this <a href="http://wiki.fasterxml.com/JacksonJsonViews">jackson view</a> will be used for serializing */
	private Class<?> viewClass;
	
	public JacksonResolution(ObjectMapper mapper, Object contents) {
		this.mapper = mapper;
		this.contents = contents;
	}
	
	@Override
	public void resolve(LoomServletRequest request, LoomServletResponse response) throws IOException, ServletException {
		if (getContentType() == null) {
			setContentType("application/json");
		}
		writeHeaders(request, response);
		if (viewClass == null) {
			mapper.writeValue(response.getWriter(), contents);
		} else {
			mapper.viewWriter(viewClass).writeValue(response.getWriter(), contents);
		}
	}

	@Override
	public String toString() {
		return new ToStringBuilder(this).append("mapper",  mapper).toString();
	}
	
	/** 
	 * Specify the  <a href="http://wiki.fasterxml.com/JacksonJsonViews">jackson view</a> to use for serializing 
	 */
	public JacksonResolution withView(Class<?> viewClass) {
		this.viewClass = viewClass;
		return this;
	}

	public ObjectMapper getMapper() {
		return mapper;
	}

	public Object getContents() {
		return contents;
	}

	@Override
	public JacksonResolution addCookie(String name, String value) {
		return (JacksonResolution) super.addCookie(name, value);
	}
	
	@Override
	public JacksonResolution addCookie(String name, String value, int maxAge) {
		return (JacksonResolution) super.addCookie(name, value, maxAge);
	}
	
	@Override
	public JacksonResolution addCookie(Cookie cookie) {
		return (JacksonResolution) super.addCookie(cookie);
	}
	
	@Override
	public JacksonResolution addHeader(String name, Object value) {
		return (JacksonResolution) super.addHeader(name, value);
	}
	
	@Override
	public JacksonResolution setHeader(String name, Object value) {
		return (JacksonResolution) super.setHeader(name, value);
	}
	
	@Override
	public JacksonResolution setHeaderIfNotPresent(String name, Object value) {
		return (JacksonResolution) super.setHeaderIfNotPresent(name, value);
	}
	
	@Override
	public JacksonResolution setCharset(String charset) {
		return (JacksonResolution) super.setCharset(charset);
	}
	
	@Override
	public JacksonResolution setContentType(String contentType) {
		return (JacksonResolution) super.setContentType(contentType);
	}
	
	@Override
	public JacksonResolution setCacheControl(CacheControl cacheControl) {
		return (JacksonResolution) super.setCacheControl(cacheControl);
	}

}
