package org.loom.resources;

import static org.loom.config.ClassNames.CLOSURE_COMPRESSOR;
import static org.loom.config.ClassNames.YUICOMPRESSOR;
import static org.loom.util.StringUtils.calculateMD5;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.security.DigestInputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Collection;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.GZIPOutputStream;

import javax.inject.Inject;

import org.apache.commons.io.IOUtils;
import org.loom.log.Log;
import org.loom.resources.compressor.ClosureJavascriptCompressorAdapter;
import org.loom.resources.compressor.NullCompressor;
import org.loom.resources.compressor.WebResourceCompressor;
import org.loom.resources.compressor.YuiCssCompressorAdapter;
import org.loom.resources.compressor.YuiJavascriptCompressorAdapter;
import org.loom.url.UrlBuilder;
import org.loom.util.ClassUtils;
import org.loom.util.FileUtils;
import org.loom.util.StringUtils;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;

import com.google.common.collect.Sets;

public abstract class AbstractWebResourceCompiler implements WebResourceCompiler {

	/** pattern of any \@include directive */
	protected static final Pattern INCLUDE_PATTERN = Pattern.compile("//\\s*@include\\s*(\"|')([^\"'\n]+)(\"|')");
	
	/** pattern of CSS url() attributes */
	protected static final Pattern URL_PATTERN = Pattern.compile("url\\([\"']?([^\"')]+)[\"']?\\)");
	
	@Inject
	protected ResourceLoader resourceLoader;
	
	/** if true, do not concatenate or obfuscate results */
	protected boolean development;
	
	/** the compressor to use */
	protected WebResourceCompressor compressor;
	
	/** the charset to use */
	protected String charset = "UTF-8";
	
	/** the type of resources processed by this compiler */
	protected WebResourceType type;
	
	private static Log log = Log
			.getLog(AbstractWebResourceCompiler.class);
	
	@Override
	public final CompilationResult compile(String webResourceBundleName, Collection<Resource> resources) {
		try {
			resources = preprocess(resources);
			return development? compileDev(resources) : compileProd(WebResourceBundle.PROD_RESOURCE_NAME, resources);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	protected CompilationResult compileDev(Collection<Resource> resources) throws IOException {
		CompilationResult result = new CompilationResult(type, resources.size(), resources);
		for (Resource resource : resources) {
			String contents = IOUtils.toString(resource.getInputStream());
			if (type == WebResourceType.CSS) {
				contents = replaceCssUrls(result, resource, contents);
			}
			result.addCompiledWebResource(createCompiledWebResource(resource.getFilename(), contents));
		}
		return result;
	}
	
	protected CompilationResult compileProd(String webResourceBundleName, Collection<Resource> resources) throws IOException {
		CompilationResult result = new CompilationResult(type, 1, resources);
		StringBuilder builder = new StringBuilder(1024);
		for (Resource resource : resources) {
			String contents = IOUtils.toString(resource.getInputStream());
			if (type == WebResourceType.CSS) {
				contents = replaceCssUrls(result, resource, contents);
			}
			builder.append(contents);
		}
		String contents = compressor.compress(webResourceBundleName, builder.toString());
		result.addCompiledWebResource(createCompiledWebResource(webResourceBundleName, contents));
		return result;
	}

	protected CompiledWebResource createCompiledWebResource(String name, String contents) throws IOException {
		
		GZIPOutputStream output = null;
		DigestInputStream input = null;
		try {
			
			// gzip and calculate MD5
			ByteArrayOutputStream os = new ByteArrayOutputStream();
			MessageDigest digest = MessageDigest.getInstance("MD5");
			input = new DigestInputStream(new ByteArrayInputStream(contents.getBytes(charset)), digest);
			output = new GZIPOutputStream(os);
			FileUtils.copy(input, output);
			output.close(); // to flush the GZIP output
			return createCompiledWebResource(name, StringUtils.toString(digest), contents, os.toByteArray());
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		} finally {
			IOUtils.closeQuietly(input);
			IOUtils.closeQuietly(output);
		}

		
	}

	protected String replaceCssUrls(CompilationResult result, Resource rootResource, String css) throws IOException {
		Matcher matcher = URL_PATTERN.matcher(css);
		int end = 0;
		StringBuilder processed = new StringBuilder(css.length());
		while (matcher.find() && end < css.length()) {
			int start = matcher.start();
			processed.append(css.substring(end, start));
			end = matcher.end();
			String url = matcher.group(1);
			
			if (url.startsWith("/")) { // prefix the webapp context
				url = new UrlBuilder(url).getURL();
			} else if (!url.startsWith("http://") && !url.startsWith("https://") && !url.startsWith("../")) {
				Resource res = rootResource.createRelative(url);
				String md5 = "0";
				try {
					md5 = calculateMD5(res);
				} catch (FileNotFoundException e) {
					// the file is not there
					log.warn("Missing CSS resource " + res.getDescription());
				}
				result.addCssResource(new CssResource(url, md5, res));
				url = "../_res/" + md5 + "/" + url;
			}
			processed.append("url('").append(url).append("')");
			
		}
		if (end < css.length()) {
			processed.append(css.substring(end));
		}
		return processed.toString();
	}

	/**
	 * Create a {@link CompiledWebResource}
	 * @param name the name of the web resource
	 * @param md5 the md5 checksum
	 * @param contents the contents of the file
	 * @param gzippedContents the gzipped contents of the file
	 * @throws IOException 
	 */
	protected abstract CompiledWebResource createCompiledWebResource(String name,
			String md5, String contents, byte[] gzippedContents) throws IOException;
/*
	protected String concatenateAndMinify(String webResourceBundleName, Collection<Resource> resources) throws IOException {
		StringBuilder builder = new StringBuilder(1024);
		for (Resource resource : resources) {
			InputStream input = null;
			input = resource.getInputStream();
			builder.append(IOUtils.toString(input));
		}
		return compressor.compress(webResourceBundleName, builder.toString());
	}

	/**
	 * Process \@include directives in the source files
	 * @throws IOException 
	 */
	protected Collection<Resource> preprocess(Collection<Resource> resources) throws IOException {
		Set<Resource> processedResources = Sets.newLinkedHashSet();
		for (Resource resource : resources) {
			addResource(processedResources, resource);
		}
		return processedResources;
	}

	/**
	 * Adds a resource after processing any \@include directives
	 * @throws IOException 
	 */
	protected void addResource(Set<Resource> processedResources, Resource resource) throws IOException {
		InputStream input = null;
		if (!resource.exists()) {
			throw new IllegalArgumentException("Resource cannot be found: " + resource.getDescription());
		}
		try {
			log.debug("Adding ", resource.getDescription());
			input = resource.getInputStream();
			String inputStr = IOUtils.toString(input);
			Matcher matcher = INCLUDE_PATTERN.matcher(inputStr);
			while (matcher.find()) {
				String location = matcher.group(2);
				Resource requiredResource = location.startsWith("classpath:") || location.startsWith("/")?
						resourceLoader.getResource(location) : resource.createRelative(location);
				addResource(processedResources, requiredResource);
			}
			processedResources.add(resource);
		} finally {
			IOUtils.closeQuietly(input);
		}
	}

	public void setResourceLoader(ResourceLoader resourceLoader) {
		this.resourceLoader = resourceLoader;
	}

	public void setDevelopment(boolean development) {
		this.development = development;
	}

	public void setType(WebResourceType type) {
		this.type = type;
		if (compressor == null) {
			if (type == WebResourceType.CSS) {
				if (ClassUtils.isPresent(YUICOMPRESSOR)) {
					log.info("Detected YUI CSS Compressor");
					setCompressor(new YuiCssCompressorAdapter()); 
				}
			} else if (type == WebResourceType.JS) {
				if (ClassUtils.isPresent(CLOSURE_COMPRESSOR)) {
					log.info("Detected Google Closures JS compressor");
					setCompressor(new ClosureJavascriptCompressorAdapter());
				} else if (ClassUtils.isPresent(YUICOMPRESSOR)) {
					log.info("Detected YUI JS Compressor");
					setCompressor(new YuiJavascriptCompressorAdapter());
				}
			}
			if (compressor == null) {
				log.warn("No ", type, " compressor found in classpath");
				compressor = new NullCompressor();
			}
		} 
	}

	public void setCompressor(WebResourceCompressor compressor) {
		this.compressor = compressor;
	}
	
}
