package org.loom.resources;

import java.io.IOException;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import javax.servlet.jsp.JspException;

import org.loom.action.support.ResourcesAction;
import org.loom.exception.HttpException;
import org.loom.tags.core.AbstractWebResourceTag;
import org.springframework.core.io.Resource;

import com.google.common.collect.Maps;

public class DefaultWebResourceBundle implements WebResourceBundle {
	
	/** the name of this instance */
	protected String name;
	
	/** the list of source resource locations */
	protected Set<Resource> resources = new LinkedHashSet<Resource>();

	/** the type of this bundle */
	protected WebResourceType type;
	
	/** the compiler to use */
	protected WebResourceCompiler compiler;

	/** the list of compiled web resources to serve. compiledWebResources.length != resources.length */
	protected Map<String, CompiledWebResource> compiledWebResources;
	
	/** 
	 * if type == CSS, this is the list of images referenced by url() attributes in the CSS files. 
	 * The key value is the name of the image url
	 * For JS bundles its value is null 
	 */
	protected Map<String, CssResource> cssResources;
	
	@Override
	public void render(AbstractWebResourceTag tag) throws JspException, IOException {
		if (compiledWebResources == null) {
			compile();
		}
		tag.getUrl().replace(ResourcesAction.BUNDLE, name);
		for (CompiledWebResource res : compiledWebResources.values()) {
			res.render(tag);
		}
	}
	
	/**
	 * this.compiledWebResources is not transient on purpose, since double instantiation 
	 * is not really a problem.
	 * We're just trying to be clean here, but without hurting performance for the common case.
	 */
	protected synchronized CompilationResult compile() {
		CompilationResult compileResult = null;
		if (compiledWebResources == null) {
			compileResult = compiler.compile(name, resources);
			compiledWebResources = Maps.newLinkedHashMap();
			for (CompiledWebResource res : compileResult.getCompiledResources()) {
				compiledWebResources.put(res.getName(), res);
			}
			if (compileResult.getCssResources() != null) {
				cssResources = Maps.newHashMapWithExpectedSize(compileResult.getCssResources().size());
				for (CssResource cssResource : compileResult.getCssResources()) {
					cssResources.put(cssResource.getName(), cssResource);
				}
			}
		}
		return compileResult;
	}
	
	public CompiledWebResource getCompiledWebResource(String name) {
		if (compiledWebResources == null) {
			compile();
		}
		CompiledWebResource result = compiledWebResources.get(name);
		if (result == null) {
			throw new HttpException(404, "CompiledWebResource with name '" + name + "' not found in bundle '" + name + "'");
		}
		return result;
	}
	
	public CssResource getCssResource(String name) {
		CssResource result = cssResources.get(name);
		if (result == null) {
			throw new HttpException(404, "CssExternalResource with name '" + name + "' not found in bundle '" + name + "'");
		}
		return result;
	}

	public void setResources(Set<Resource> resources) {
		this.resources = resources;
		type = WebResourceType.identify(resources.iterator().next());
	}
	
	public WebResourceType getType() {
		return type;
	}

	public void setType(WebResourceType type) {
		this.type = type;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setCompiler(WebResourceCompiler compiler) {
		this.compiler = compiler;
	}

	public String getName() {
		return name;
	}

	public Map<String, CompiledWebResource> getCompiledWebResources() {
		return compiledWebResources;
	}

	public Map<String, CssResource> getCssResources() {
		return cssResources;
	}
	
}
