package org.loom.servlet;

import java.util.Locale;

import javax.inject.Inject;
import javax.inject.Singleton;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.loom.config.LocaleResolver;
import org.loom.i18n.MessagesRepository;
import org.loom.i18n.MessagesRepositoryFactory;
import org.loom.mapping.ActionMappingRepository;
import org.loom.servlet.names.RequestAttributeNames;
import org.loom.servlet.params.FileParameterFactory;
import org.loom.servlet.params.ServletRequestParameters;
import org.loom.servlet.scope.FlashContextFactory;

@Singleton
public class LoomServletRequestFactory {
	
	@Inject
	private ServletContext servletContext;
	
	/**
	 * creates MessagesRepository instances with the locale-aware contents adapted to the
	 * current request locale
	 */
	@Inject
	private MessagesRepositoryFactory messagesRepositoryFactory;
	
	/**
	 * calculates the Locale to use for the current request
	 */
	@Inject
	private LocaleResolver localeResolver;
	
	/**
	 * Creates a flash context for the current request
	 */
	@Inject
	private FlashContextFactory flashContextFactory;

	/** creates uploaded file items (adapter for commons-fileupload) */
	@Inject
	private FileParameterFactory fileParameterFactory;

	@Inject 
	private ActionMappingRepository actionMappingRepository;
	
	/** the default charset to be used for text/*, application/x-json and application/x-javascript responses, defaults to UTF-8 */
	private String defaultCharset = "UTF-8";
	
	public LoomServletRequest getOrCreateInstance(ServletRequest parent) {
		LoomServletRequest instance = (LoomServletRequest) parent.getAttribute(RequestAttributeNames.REQUEST);
		if (instance == null) {
			instance = createInstance(parent);
		}
		return instance;
	}
	
	public void clearInstance(ServletRequest request) {
		request.removeAttribute(RequestAttributeNames.REQUEST);
		LoomServletRequestImpl.clearThreadLocal();
	}

	private LoomServletRequest createInstance(ServletRequest req) {
		HttpServletRequest request = (HttpServletRequest) req;
		LoomServletRequestImpl instance = new LoomServletRequestImpl(request, createData(request));
		req.setAttribute(RequestAttributeNames.REQUEST, instance);
		LoomServletRequestImpl.setThreadLocal(instance);
		return instance;
	}

	private LoomServletRequestData createData(HttpServletRequest request) {
		LoomServletRequestData data = new LoomServletRequestData();
		data.setDefaultCharset(defaultCharset);
		data.setServletContext(servletContext);
		data.setMessagesRepository(getMessagesRepository(request));
		data.setFlashContextFactory(flashContextFactory);
		data.setServletRequestParameters(new ServletRequestParameters(defaultCharset, request, fileParameterFactory));
		data.setActionMappingRepository(actionMappingRepository);
		return data;
	}

	private MessagesRepository getMessagesRepository(HttpServletRequest request) {
		Locale locale = localeResolver.getLocale(request);
		if (locale == null) {
			locale = request.getLocale();
		}
		return messagesRepositoryFactory.getRepository(locale);
	}
	
	/**
	 * Returns the instance of LoomServletResponse if one already exists, or creates a new one if not
	 */
    public LoomServletResponse getResponseInstance(LoomServletRequest request, ServletResponse parent) {
    	LoomServletResponseImpl response = (LoomServletResponseImpl) request.getAttribute(RequestAttributeNames.RESPONSE);
    	if (response != null) {
    		return response;
    	}

        // set the same Locale that has been calculated for the request object
    	response = new LoomServletResponseImpl(parent, initResponseData(request.getLocale()));
    	request.setAttribute(RequestAttributeNames.RESPONSE, response);
        return response;

    }
    
    private LoomServletResponseData initResponseData(Locale locale) {
    	LoomServletResponseData data = new LoomServletResponseData();
    	data.setCharset(defaultCharset);
    	data.setLocale(locale);
    	return data;
    }
    
	public void setServletContext(ServletContext servletContext) {
		this.servletContext = servletContext;
	}

	public void setMessagesRepositoryFactory(
			MessagesRepositoryFactory messagesRepositoryFactory) {
		this.messagesRepositoryFactory = messagesRepositoryFactory;
	}

	public void setLocaleResolver(LocaleResolver localeResolver) {
		this.localeResolver = localeResolver;
	}

	public void setFlashContextFactory(FlashContextFactory flashContextFactory) {
		this.flashContextFactory = flashContextFactory;
	}

	public void setDefaultCharset(String defaultCharset) {
		this.defaultCharset = defaultCharset;
	}

	public ServletContext getServletContext() {
		return servletContext;
	}

	public MessagesRepositoryFactory getMessagesRepositoryFactory() {
		return messagesRepositoryFactory;
	}

	public LocaleResolver getLocaleResolver() {
		return localeResolver;
	}

	public FlashContextFactory getFlashContextFactory() {
		return flashContextFactory;
	}

	public String getDefaultCharset() {
		return defaultCharset;
	}

	public void setFileParameterFactory(FileParameterFactory fileParameterFactory) {
		this.fileParameterFactory = fileParameterFactory;
	}
	
	/**
	 * Returns the instance of LoomServletRequestWrapper, if one exists, or
	 * instantiates one if not.
	 * If the instance is not found and create is false, throws an exception.
	 * @deprecated use either getInstance or getOrCreateInstance instead
	 * /
	public static LoomServletRequestImpl getInstance(ServletRequest parent, boolean create) {
		return create? getOrCreateInstance(parent) : getInstance(parent);
	}*/

}
