/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.tags.core.FormTag;
import org.loom.util.PropertyUtils;
import org.tldgen.annotations.Attribute;

/**
 * Parent class for any form field associated to a form parameter.
 * 
 * @author icoloma
 */
public abstract class AbstractFormFieldTag extends AbstractHtmlTag {
	
	/** the normalized property path. Ths is the same as name, but removing any indexes for collections or maps */
	private String normalizedName;

	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-tabindex">HTML attribute</a> */
	@Attribute
	private Integer tabindex;

	/** <a href="http://www.w3.org/TR/html4/struct/objects.html#adef-alt">HTML attribute</a> */
	@Attribute
	private String alt;

	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-disabled">HTML attribute</a> */
	@Attribute
	private Boolean disabled;

	/**
	 * Internal value initialized by child classes 
	 * <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-type-INPUT">input</a> 
	 */
	private String type;
	
	/** the enclosing form tag */
	private FormTag form;

	/** If not null, the provided value will be used as field label. Defaults to the name attribute */
	@Attribute
	private String label;
	
	/** If not null, this value will be used as help line */
	@Attribute
	private String helpLine;
	
	@Override
	protected void initTag() {
		super.initTag();
		this.form = request.getCurrentForm();
		if (getName() != null && form == null) {
			throw new IllegalArgumentException(getClass().getSimpleName() + " must be nested inside a <l:form> tag");
		}
	}
	
	/**
	 * Prints this instance attributes
	 * @throws JspException 
	 */
	@Override
	protected void printParentAttributes() throws IOException, JspException {
		super.printParentAttributes();
		out.printAttribute("alt", repository.getString(alt));
		out.printAttribute("tabindex", tabindex);
		out.printAttribute("type", type);
		if (isDisabled()) {
			out.print(" disabled");
		}
	}
	
	/** 
	 * HTML attribute with different meanings:
	 * <ul>
	 * <li><a href="http://www.w3.org/TR/html4/interact/forms.html#adef-name-BUTTON">button, textarea</a></li>
	 * <li><a href="http://www.w3.org/TR/html4/interact/forms.html#adef-name-SELECT">select</a></li>
	 * <li><a href="http://www.w3.org/TR/html4/interact/forms.html#adef-name-INPUT">input</a></li>
	 * </ul>
	 * This is the property path inside the action that will be used to get and set the field value. 
	 */
	@Override
	@Attribute(required=true)
	public void setName(String name) {
		super.setName(name);
		this.normalizedName = PropertyUtils.normalize(name);
	}
	
	public void setDisabled(boolean disabled) {
		this.disabled = disabled;
	}
	
	public boolean isDisabled() {
		return disabled != null? disabled : form != null && form.isDisabled();
	}

	public void setAlt(String alt) {
		this.alt = alt;
	}

	public void setType(String type) {
		this.type = type;
	}

	public FormTag getForm() {
		return form;
	}

	public void setForm(FormTag form) {
		this.form = form;
	}

	public String getLabel() {
		return label == null? getNormalizedName() : label;
	}
	
	public String getNormalizedName() {
		return normalizedName;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public void setTabindex(Integer tabindex) {
		this.tabindex = tabindex;
	}
	
	public Integer getTabindex() {
		return tabindex;
	}

	public String getHelpLine() {
		return helpLine;
	}

	public void setHelpLine(String helpLine) {
		this.helpLine = helpLine;
	}
}
