/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags;

import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.lang.StringUtils;


/**
 * Concatenates CSS styles, separating with spaces the different styles.
 * Admits null as a css class (that will not get added) and ignores a css 
 * class if it is already present.
 * @author Ignacio Coloma
 */
public class CssBuilder {
	
	/** the list of CSS classes already present */
	private Set<String> contents = new TreeSet<String>();
	
	/** the calculated css result */
	private String cachedResult;
	
	/** true if the result must be recalculated (a css style has been added after the last call to getCss()) */
	private boolean dirty;
	
	public CssBuilder() {
	}
	
	/**
	 * Initializes a CssBuilder with the given css classes
	 * @param cssClasses the list of css classes to be included by default
	 */
	public CssBuilder(String... cssClasses) {
		for (String css : cssClasses) {
			add(css);
		}
	}
	
	/**
	 * @param includedClassNames the css class names that should be included in the copy. The rest are discarded.
	 * @return a copy of this CssBuilder instance, including only the filtered classes
	 */
	public CssBuilder filter(Set<String> includedClassNames) {
		CssBuilder copy = new CssBuilder();
		for (String cssClassName : contents) {
			if (includedClassNames.contains(cssClassName)) {
				copy.add(cssClassName);
			}
		}
		return copy;
	}

	/**
	 * Appends a css class if the class is != null
	 * @param cssClass the CSS class to be added. If null, it will be ignored.
	 */
	public CssBuilder add(String cssClass) {
		if (cssClass != null) {
			dirty = contents.add(cssClass) || dirty;
		}
		return this;
	}
	
	/**
	 * Adds all the CSS classes included in another instance of {@link CssBuilder} into this one.
	 * @param other the CSS builder to be added. 
	 */
	public CssBuilder addAll(CssBuilder other) {
		dirty = contents.addAll(other.contents) || dirty;
		return this;
	}
	
	/**
	 * @return the accumulated CSS, null if empty
	 */
	public String getCss() {
		if (!dirty) {
			return cachedResult;
		}
		dirty = false;
		StringBuilder builder = new StringBuilder();
		for (Iterator<String> i = contents.iterator(); i.hasNext();) {
			String css = i.next();
			builder.append(css);
			if (i.hasNext()) {
				builder.append(' ');
			}
		}
		cachedResult = builder.toString();
		return cachedResult;
	}
	
	/**
	 * Clears this cssBuilder instance
	 */
	public void clear() {
		contents.clear();
		cachedResult = null;
		dirty = false;
	}

	/**
	 * @return true if this css builder does not have any css styles yet
	 */
	public boolean isEmpty() {
		return contents.isEmpty();
	}

	/**
	 * Clears the current css classes and add a new one.
	 * @param css
	 */
	public void setCss(String css) {
		clear();
		if (!StringUtils.isEmpty(css)) {
			add(css);
		}
	}
	
	/**
	 * @return the number of CSS classes added to this instance
	 */
	public int size() {
		return contents.size();
	}
	
	/**
	 * @return true if this CSS builder contains the provided css class name. This is (supposedly) 
	 * faster than querying the concatenated string, specially when the CSS is not found. 
	 */
	public boolean contains(String cssClassName) {
		return contents.contains(cssClassName);
	}

}
