/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags;

import java.io.IOException;
import java.io.Writer;

/**
 * Wrapper around the JspWriter class
 * 
 * @author icoloma
 * @param <T> the Writer class being used, usually a JspWriter
 */
public interface WriterWrapper<T extends Writer> {

	/**
	 * Prints a tag attribute and value, if not null
	 * @param name name of the attribute
	 * @param value value of the attribute. If null, nothing will be written
	 * @return this same instance
	 */
	public WriterWrapper<T> printAttribute(String name, Object value) throws IOException;

	/**
	 * Print a String
	 * @param s the text to be written
	 * @return this same instance
	 */
    public WriterWrapper<T> print(CharSequence s) throws IOException;
    
    /**
     * Print ant Object, invoking its toString() method
     * @param o the object to be written
     * @return this same instance
     */
    public WriterWrapper<T> print(Object o) throws IOException;

    /**
     * @return the true writer instance
     */
    public Writer getWriter();

    /**
     * Prints a hidden form field if its value is not null
     * @param name name of the field
     * @param value value of the field. If null, nothing will be written
     * @throws IOException 
     */
	public WriterWrapper<T> printHiddenField(String name, Object value) throws IOException;

	/**
	 * Write a javascript start tag
	 */
	public WriterWrapper<T> startJavascript() throws IOException;

	/**
	 * Write a javascript end tag
	 */
	public WriterWrapper<T> endJavascript() throws IOException;
	
	/**
	 * Flush the written contents
	 */
	public void flush() throws IOException;
	
	/**
	 * Start an HTML tag.
	 * This method will write the starting tag complete with all the provided attributes.
	 * The attributes must be in pairs (name, value)
	 * Example of use: start("a", "href", "foo.html", "class", "bar") will write 
	 * &lt;a href="foo.html" class="bar">
	 * If any of the provided attribute values is null, it will be ignored. 
	 * @throws IllegalArgumentException if the number of attribute names and values do 
	 * not match.  
	 */
	public WriterWrapper<T> start(String tag, Object... attributes) throws IOException;
	
	/**
	 * Start and end a bodyless HTML tag.
	 * This method will write the tag complete with all the provided attributes.
	 * The attributes must be in pairs (name, value)
	 * Example of use: startEnd("img", "href", "foo.html", "class", "bar") will write 
	 * &lt;img href="foo.html" class="bar"/>
	 * If any of the provided attribute values is null, it will be ignored. 
	 * @throws IllegalArgumentException if the number of attribute names and values do 
	 * not match.  
	 */
	public WriterWrapper<T> startEnd(String tag, Object... attributes) throws IOException;
	
	/**
	 * Ends a HTML tag
	 * @param tag
	 */
	public WriterWrapper<T> end(String tag) throws IOException;
	
	
}
