/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.mapping.Event;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.tags.AbstractFormFieldTag;
import org.loom.tags.Button;
import org.loom.util.PropertyUtils;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.ExcludeProperties;
import org.tldgen.annotations.Tag;


/**
 * <a href="http://www.w3.org/TR/html4/interact/forms.html#submit-button">Submit</a>, 
 * <a href="http://www.w3.org/TR/html4/interact/forms.html#reset-button">Reset</a> or 
 * <a href="http://www.w3.org/TR/html4/interact/forms.html#push-button">Push</a> button. This component will generate a input type="{ button | reset | submit}" tag. 
 * The <code>value</code> property is only used to print the button text.
 * @author icoloma
 */
@Tag(
		dynamicAttributes=true,
		bodyContent=BodyContent.EMPTY,
		example=
			"&lt;l:button/>\n" +
			"&lt;l:button value=\"save\" event=\"save\"/>"
	)
@ExcludeProperties({ "accept", "label", "name" })
public class ButtonTag extends AbstractFormFieldTag implements Button {

	/** supported button types */
	private enum Type { BUTTON, RESET, SUBMIT }
	
	/** the button type, default submit */
	private Type type = Type.SUBMIT;
	
	/**
	 * <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-value-INPUT">HTML attribute</a>. 
	 * The text that will appear on the submit button. By default, [ActionName].[event] will be used.
	 */
	@Attribute
	private String value;
	
	/** 
	 * When a submit button with this property set is clicked, the form will be submitted to 
	 * this event instead of the one specified in the container form. The browser (javascript) validations 
	 * will still be the ones related to the original form event.
	 */
	@Attribute
	private String event; 

	@Override
	protected void initTag() {
		super.initTag();
		if (getForm() == null) {
			throw new IllegalArgumentException("Buttons can only be used inside a form");
		}
		Event e = event == null? getForm().getTargetEvent() : getForm().getTargetAction().getEvent(event);
		e.decorate(this);
	}
	
	@Override
	public void doTagImpl() throws JspException, IOException {
		// guess the value attribute
		if (value == null && Type.SUBMIT == type) {
			value = PropertyUtils.concatPropertyPath(getForm().getTargetAction().getShortName(), event != null? event : getForm().getTargetEvent().getName());
		}
		
		out.print("<input");
		if (event != null) {
			out.printAttribute("name", RequestParameterNames.OVERRIDE_EVENT_PREFIX + event);
		}
		out.printAttribute("type", type);
		out.printAttribute("value", repository.guessString(value));
		printParentAttributes();
		out.print(" />");
	}
	
	public void setValue(String value) {
		this.value = value;
	}

	public String getType() {
		return type.toString();
	}

	/** the button type, possible values are "button", "reset" and "submit" (default)  */
	@Override
	@Attribute
	public void setType(String type) {
		this.type = Type.valueOf(type.toUpperCase());
		addCssClass(type);
	}

	public void setEvent(String event) {
		this.event = event;
	}
	
}
