/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.action.support.ResourcesAction;
import org.loom.url.UrlBuilder;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * <a href="http://www.w3.org/TR/html4/interact/scripts.html#edef-SCRIPT">HTML script tag</a>. 
 * Javascript files can be delivered one by one or bundled together as a web resource
 * configured using your DI engine.
 * @author icoloma
 *
 */
@Tag(
		dynamicAttributes=true, 
		bodyContent=BodyContent.SCRIPTLESS,
		example=
			"&lt;l:script src=\"/js/IE7.js\" ieLessThan=\"7\"/>\n" +
			"&lt;l:script action=\"Resources\" event=\"pull\"/>\n" +
			"&lt;l:script resource=\"admin-javascript-resources\"/>"
	)
public class ScriptTag extends AbstractWebResourceTag {

	/** <a href="http://www.w3.org/TR/html4/struct/links.html#adef-charset">HTML attribute</a> */
	@Attribute
	private String charset;
	
	/** <a href="http://www.w3.org/TR/html4/interact/scripts.html#adef-defer">HTML attribute</a> */
	@Attribute
	private Boolean defer;
	
	/** <a href="http://www.w3.org/TR/html4/interact/scripts.html#adef-language">HTML attribute</a> */
	@Attribute
	private String language;
	
	public ScriptTag() {
		this.setType("text/javascript");
	}
	
	@Override
	public void printTag() throws JspException, IOException {
		// if this is a link to the MessagesRepository, generate automatically the MD5 param
		UrlBuilder urlBuilder = getUrlBuilder();
		if (urlBuilder.getActionMapping() != null && ResourcesAction.class == urlBuilder.getActionMapping().getActionClass() && urlBuilder.getEvent().isDefault()) {
			urlBuilder.replace(ResourcesAction.MD5, getMessagesRepository().getMD5());
		}
		
		out.print("<script");
		out.printAttribute("charset", charset);
		out.printAttribute("defer", defer);
		out.printAttribute("language", language);
		if (!urlBuilder.isEmpty()) {
			out.printAttribute("src", urlBuilder.getURL());
		}
		out.printAttribute("type", getType());
		out.print(">");
		out.print(getContents());
		out.print("</script>");
	}
	
	/** 
	 * <a href="http://www.w3.org/TR/html4/struct/links.html#adef-href">HTML attribute</a>
	 * The target url, or alternatively use the action and event properties 
	 */
	@Attribute
	public void setSrc(String href) {
		setRawUrl(href);
	}

	public void setCharset(String charset) {
		this.charset = charset;
	}

	public void setDefer(Boolean defer) {
		this.defer = defer;
	}

	public void setLanguage(String language) {
		this.language = language;
	}

}
