package org.loom.util;

import java.lang.reflect.Method;
import java.util.Date;

import org.apache.commons.lang.StringUtils;
import org.joda.time.Period;
import org.joda.time.PeriodType;
import org.loom.i18n.Message;
import org.loom.i18n.MessagesRepository;


public class DateUtils {
	
	/** the metadata of Duration getters to use */
	private static Method[] getterMetadata;
	
	/** durations per unit, in seconds */
	private static int unitDurations[] = new int[] { 
		365 * 24 * 60 * 60, // years 
		30 * 24 * 60 * 60,  // months
		7 * 24 * 60 * 60,   // weeks
		24 * 60 * 60,       // days
		60 * 60,            // hours
		60,                 // minutes
		1					// seconds
	};
	
	static {
		try {
			getterMetadata = new Method[7];
			getterMetadata[0] = Period.class.getMethod("getYears");
			getterMetadata[1] = Period.class.getMethod("getMonths");
			getterMetadata[2] = Period.class.getMethod("getWeeks");
			getterMetadata[3] = Period.class.getMethod("getDays");
			getterMetadata[4] = Period.class.getMethod("getHours");
			getterMetadata[5] = Period.class.getMethod("getMinutes");
			getterMetadata[6] = Period.class.getMethod("getSeconds");
		} catch (NoSuchMethodException e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Format a Date in minutes/hours/days etc, relative to today.
	 * Equivalent to <a href="http://api.rubyonrails.org/classes/ActionView/Helpers/DateHelper.html#M001696">DateHelper.time_ago_in_words</a> in Ruby
	 * and Date.humanized in format.js
	 */	
	public static String humanized(MessagesRepository repository, Date value) {
		Date d1;
		Date d2 = new Date();
		boolean future;
		if (value.after(d2)) {
			d1 = value;
			future = true;
		} else {
			d1 = d2;
			d2 = value;
			future = false;
		}
		
		String[] unitNames = StringUtils.split(repository.getString("loom.format.timeUnitNames"), ' ');
		String[] singularUnitNames = StringUtils.split(repository.getString("loom.format.singularTimeUnitNames"), ' ');

		Period period = new Period(d2.getTime(), d1.getTime(), PeriodType.yearMonthDayTime());
		int amount = 0;
		String unitName;
		String messageKey = future? "loom.format.future" : "loom.format.past";
		for (int i = 0; /*i < getterMetadata.length*/; i++) {
			Method getter = getterMetadata[i];
			amount = (Integer) ClassUtils.invoke(period, getter);
			if (amount > 0 || i == getterMetadata.length - 1) {
				unitName = amount == 1? singularUnitNames[i] : unitNames[i];
				if (i == getterMetadata.length - 1) {
					messageKey += ".seconds";
				}
				break;
			}
		}
		
		if (amount == 0) {
			amount = period.getSeconds();
		}
		
		Message message = new Message(messageKey);
		message.addArg("amount", amount);
		message.addArg("unitName", unitName);
		return repository.translateMessage(message);
	}
	
	/**
	 * Format a lapse of time in seconds/minutes/hours/days.  
	 * Equivalent to <a href="http://api.rubyonrails.org/classes/ActionView/Helpers/DateHelper.html#M001695">DateHelper.distance_of_time_in_words</a> in Ruby
	 * and Date.toDuration().humanized() in format.js
	 * @param value interval to format, in milliseconds
	 * @param maxUnits the maximum number of units to use (years, months, days, etc).
	 */
	public static String humanizedDuration(MessagesRepository repository, long value, int maxUnits) {
		String[] unitNames = StringUtils.split(repository.getString("loom.format.timeUnitNames"), ' ');
		String[] singularUnitNames = StringUtils.split(repository.getString("loom.format.singularTimeUnitNames"), ' ');
		StringBuilder sb = new StringBuilder();
		long v = value / 1000;
		int unitsCount = 0;
		for (int i = 0; i < unitDurations.length && unitsCount < maxUnits; i++) {
			int duration = unitDurations[i];
			int unitAmount = (int)(v / duration);
			if (unitAmount > 0 || (sb.length() == 0 && i == unitDurations.length - 1)) {
				if (sb.length() > 0) {
					sb.append(" ");
				}
				String unitName = unitAmount == 1? singularUnitNames[i] : unitNames[i];
				sb.append(unitAmount).append(" ").append(unitName);
				unitsCount++;
			}
			v -= unitAmount * duration;
		}
		return sb.toString();
	}
	
}
