/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.util;

import java.util.BitSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Replace invalid characters of a string.
 * Escapes quotation (") and start of tag character (<) to prevent manipulation of tag attributes 
 * and elements in the HTML code.
 * For details on this, see:
 * http://code.google.com/docreader/#p(doctype)s(doctype)t(ArticlesXSS)
 * 
 * @author icoloma
 */
public class HtmlSanitizer {

	/** set of illegal attribute characters */
	private BitSet illegalChars;
	
	/** matches any tag */
	private Pattern tagPattern = Pattern.compile("<\\/?([^<>]+)>");

	private static HtmlSanitizer instance = new HtmlSanitizer();
	
	public HtmlSanitizer() {
		illegalChars = new BitSet();
		illegalChars.set('"'); // used to close (inside an attribute)
		illegalChars.set('<'); // used to start a tag (inside HTML contents)
	}
	
	/**
	 * Replace illegal characters in a value.
	 * This method is thought to be as fast as possible. Most calls will 
	 * not need any character substitution, so it is skipped if possible
	 */
	protected String sanitizeImpl(String value) {
		if (value != null) {
			int length = value.length();
			for (int i = 0; i < length; i++) {
				char c = value.charAt(i);
				if (illegalChars.get(c)) { // contains character
					return replaceIllegalAttributeChars(value, i);
				}
			}
		}
		return value;
	}

	
	/**
	 * Replace all illegal characters with valid string representations
	 * @param index indfex of the first failed character
	 */
	protected String replaceIllegalAttributeChars(String value, int index) {
		int length = value.length();
		StringBuilder builder = new StringBuilder(length + 16);
		builder.append(value, 0, index);
		for (int i = index; i < length; i++) {
			char c = value.charAt(i);
			if (c == '<') {
				builder.append("&lt;");
			} else if (c == '"') {
				builder.append("&quot;");
			} else {
				builder.append(c);
			}
		}
		return builder.toString();
	}

	/**
	 * Removes all tags from the supplied String argument.
	 */
	public static String stripTags(String value) {
		if (value == null) {
			return value;
		}
		Matcher matcher = instance.tagPattern.matcher(value);
		return matcher.replaceAll("");
	}
	
	public static String sanitize(String s) {
		return instance.sanitizeImpl(s);
	}
	
}
