/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.util;

import java.util.HashMap;
import java.util.Map;

/**
 * Guess the mime type of a file, given its extension. Useful when the browser did 
 * not provide it. 
 * This is a simplified list, more complete lists are available e.g. in the apache sources.
 * @author icoloma
 */
public class MimeUtils {

	/** lista de tipos mime conocidos */
	private static Map<String, String> typesMap;

	static {
		typesMap = new HashMap<String, String>();
		typesMap.put("bin", "application/octet-stream");
		typesMap.put("bmp", "image/bmp");
		typesMap.put("css", "text/css");
		typesMap.put("doc", "application/msword");
		typesMap.put("dot", "application/msword");
		typesMap.put("dtd", "application/xml-dtd");
		typesMap.put("exe", "application/octet-stream");
		typesMap.put("gif", "image/gif");
		typesMap.put("gz", "application/x-gzip");
		typesMap.put("htm", "text/html");
		typesMap.put("html", "text/html");
		typesMap.put("jpe", "image/jpeg");
		typesMap.put("jpeg", "image/jpeg");
		typesMap.put("jpg", "image/jpeg");
		typesMap.put("js", "application/x-javascript");
		typesMap.put("json", "application/json");
		typesMap.put("mpp", "application/vnd.ms-project");
		typesMap.put("odg", "application/vnd.oasis.opendocument.graphics");
		typesMap.put("odp", "application/vnd.oasis.opendocument.presentation");
		typesMap.put("ods", "application/vnd.oasis.opendocument.spreadsheet");
		typesMap.put("odt", "application/vnd.oasis.opendocument.text");
		typesMap.put("pdf", "application/pdf");
		typesMap.put("ppt", "application/vnd.ms-powerpoint");
		typesMap.put("png", "image/png");
		typesMap.put("rdf", "application/rdf+xml");
		typesMap.put("tar", "application/x-tar");
		typesMap.put("tiff", "image/tiff");
		typesMap.put("txt", "text/plain");
		typesMap.put("xhtml", "application/xhtml+xml");
		typesMap.put("xls", "application/vnd.ms-excel");
		typesMap.put("xlt", "application/vnd.ms-excel");
		typesMap.put("xml", "application/xml");
		typesMap.put("xsl", "application/xml");
		typesMap.put("zip", "application/zip");
		typesMap.put("mp2", "video/mpeg");
		typesMap.put("mpa", "video/mpeg");
		typesMap.put("mpe", "video/mpeg");
		typesMap.put("mpeg", "video/mpeg");
		typesMap.put("mpg", "video/mpeg");
		typesMap.put("mov", "video/quicktime");
		typesMap.put("qt", "video/quicktime");
		typesMap.put("asf", "video/x-ms-asf");
		typesMap.put("asx", "video/x-ms-asf");
		typesMap.put("avi", "video/x-msvideo");
		typesMap.put("movie", "video/x-sgi-movie");
		typesMap.put("wmv", "video/x-ms-wmv");
		typesMap.put("flv", "video/x-flv");
	}
	
	/**
	 * @param mimeType The MIME Content-Type to consider
	 * @return true if the provided MIME type requires a charset
	 */
	public static boolean requiresCharset(String mimeType) {
		return mimeType.startsWith("text/") || mimeType.equals("application/x-javascript") || mimeType.equals("application/json"); 
	}

	/**
	 * @return the MIME type of a file, null if unknown
	 */
	public static String guessMimeType(String fileName) {
		if (fileName == null)
			return null;
		String name = fileName.toLowerCase();
		int pos = name.lastIndexOf('.');
		if (pos == -1)
			return null;
		String suffix = name.substring(pos + 1);
		return (String) typesMap.get(suffix);
	}

}
