/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.util;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;

/**
 * Hold a list of patterns that can be matched against a URL to get if they match or not.
 * By default it recognizes filenames with version numbers in it, given that it does not include "SNAPSHOT" in its name.
 * @author icoloma
 */
public class UrlMatcher {

	/** regex that matches versioned file paths or file names */
	//public static final String[] VERSIONED_RESOURCES = new String[] { "-[\\d.]+.*/", "-[\\d.]+\\.\\w+$" };
	
	/** regex that matches *-SNAPSHOT file paths */
	//public static final String[] SNAPSHOT_RESOURCES = new String[] { "SNAPSHOT" };
	
	/** the list of included patterns, default to VERSIONED_PATH */
	private List<Pattern> include;
	
	/** the list of excluded patterns, default to SNAPSHOT_PATH */
	private List<Pattern> exclude;
	
	public UrlMatcher() {
	}
	
	public UrlMatcher(String[] include, String[] exclude) {
		this.include = toPatterns(include);
		this.exclude = toPatterns(exclude);
	}
	
	/**
	 * Set the list of regex patterns that would resolve as a match
	 * @param patterns
	 */
	public static List<Pattern> toPatterns(String... patterns) {
		if (patterns == null) {
			return null;
		}
		
		List<Pattern> newPatterns = new ArrayList<Pattern>();
		for (String pattern : patterns) {
			newPatterns.add(Pattern.compile(pattern, Pattern.CASE_INSENSITIVE));
		}
		return newPatterns;
	}
	
	/**
	 * Return true if the request URI (without parameters) resolves as a match
	 */
	public boolean matches(HttpServletRequest request) {
		return matches(request.getServletPath());
	}
	
	public boolean matches(String uri) {
		
		boolean matches = false;
		if (include != null) {
			for (Pattern pattern : include) {
				if (pattern.matcher(uri).find()) {
					matches = true;
					break;
				}
			}
		}
		
		if (exclude != null && matches) {
			for (Pattern pattern : exclude) {
				if (pattern.matcher(uri).find()) {
					matches = false;
					break;
				}
			}
		}
		
		return matches;
	}
	
	public void setInclude(List<Pattern> include) {
		this.include = include;
	}
	
	public void setExclude(List<Pattern> exclude) {
		this.exclude = exclude;
	}
	
	public void setInclude(String[] include) {
		setInclude(toPatterns(include));
	}
	
	public void setExclude(String[] exclude) {
		setExclude(toPatterns(exclude));
	}
	
}
