/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.validator;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.lang.ArrayUtils;


/**
 * Filter text according to a set of rules (ant-style). It can be used to text if a text matches to a 
 * set of include and exclude patterns.
 * @author icoloma
 */
public class PathMatcher implements Cloneable {

	private List<PathMatcherRule> rules = new ArrayList<PathMatcherRule>();

	public PathMatcher() {
	}
	
	public PathMatcher(PathMatcher parent) {
		this(parent, null, null);
	}
	
	public PathMatcher(String[] on, String[] except) {
		this(null, on, except);
	}
	
	/**
	 * @param parent the event filter of the parent of this property, for nested property paths. May be null.
	 * @param on the events on which this event filter will trigger
	 * @param except the events on which this event filter will not trigger
	 */
	public PathMatcher(PathMatcher parent, String[] on, String[] except) {
		if (!ArrayUtils.isEmpty(on) || !ArrayUtils.isEmpty(except)) {
			addRule(new PathMatcherRule(on, except));
		}
		if (parent != null) {
			addAll(parent);
		}
	}
	
	/**
	 * Add new on/except constraints to this filter instance
	 * @param on
	 * @param except
	 */
	public void setConstraints(String[] on, String[] except) {
		addRule(new PathMatcherRule(on, except));
	}
	
	protected void addAll(PathMatcher other) {
		rules.addAll(other.rules);
	}
	
	public boolean matches(String event) {
		for (PathMatcherRule filter : rules) {
			if (!filter.matches(event)) {
				return false;
			}
		}
		return true;
	}

	public PathMatcher addRule(PathMatcherRule rule) {
		rules.add(rule);
		return this;
	}
	
	@Override
	public String toString() {
		if (rules.isEmpty()) {
			return "all";
		}
		StringBuilder builder = new StringBuilder(); 
		builder.append("{");
		for (Iterator<PathMatcherRule> i = rules.iterator(); i.hasNext();) {
			PathMatcherRule filter = i.next();
			builder.append(filter);
			if (i.hasNext()) {
				builder.append(", ");
			}
		}
		builder.append("}");
		return builder.toString();
	}

	
}
